/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 * Copyright (c) 2002-2005 LinuxMagic Inc. All Rights Reserved.
 *
 * This file contains Original Code and/or Modifications of Original Code as
 * defined in and that are subject to the Free Source Code License Version
 * 1.0 (the 'License'). You may not use this file except in compliance with
 * the License. Please obtain a copy of the License at:
 *
 * http://www.linuxmagic.com/opensource/licensing/FSCL.txt
 *
 * and read it before using this file.
 *
 * The Original Code and all software distributed under the License are
 * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESS OR IMPLIED, AND LINUXMAGIC HEREBY DISCLAIMS ALL SUCH WARRANTIES,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT. Please see
 * the License for the specific language governing rights and limitations
 * under the License."
 *
 * Please read the terms of this license carefully. By using or downloading
 * this software or file, you are accepting and agreeing to the terms of this
 * license with LinuxMagic Inc. If you are agreeing to this license on behalf
 * of a company, you represent that you are authorized to bind the company to
 * such a license. If you do not meet this criterion or you do not agree to
 * any of the terms of this license, do NOT download, distribute, use or alter
 * this software or file in any way.
 *
 * Author(s): Burton Samograd <burton@wizard.ca>
 *            Josh Wilsdon <josh@wizard.ca>
 *
 * CVS Id: $Header: /web/programmers/cvs/magicmail-backend/common/base64.c,v 1.3 2005/12/19 21:13:49 josh Exp $
 *
 * DO NOT MODIFY WITHOUT CONSULTING THE LICENSE
 */

#include <assert.h>
#include <stdlib.h>
#include <string.h>

static char *b64alpha = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
#define B64PAD '='

int b64decode(char *in, char **outstr)
{
    char *out = NULL;
    char *s = NULL;
    int inlen;
    int outlen;
    int i, j;
    unsigned char a[4];
    unsigned char b[3];

    inlen = strlen(in);
    outlen = inlen + 2;
    out = malloc(outlen * sizeof(char));
    if (out == NULL) {
        return -1;
    }
    s = out;

    for (i = 0; i < inlen; i += 4) {
        for (j = 0; j < 4; j++) {
            if (((i + j) < inlen) && (in[i + j] != B64PAD)) {
                char *tmp;

                tmp = strchr(b64alpha, in[i + j]);
                if (tmp == NULL) {
                    free(out);
                    return 1;
                }
                a[j] = tmp - b64alpha;
            } else {
                a[j] = 0;
            }
        }

        b[0] = (a[0] << 2) | (a[1] >> 4);
        b[1] = (a[1] << 4) | (a[2] >> 2);
        b[2] = (a[2] << 6) | (a[3]);

        *s++ = b[0];

        if (in[i + 1] == B64PAD) {
            break;
        }
        *s++ = b[1];

        if (in[i + 2] == B64PAD) {
            break;
        }
        *s++ = b[2];
    }
    /* null terminate */
    assert(s != NULL);
    *s = '\0';

    *outstr = out;
    return 0;
}

int b64encode(char *in, char **outstr)
{
    unsigned char a, b, c;
    char *s;
    char *out = NULL;
    char outlen;
    int inlen;
    int i;

    inlen = strlen(in);

    if (inlen == 0) {
        *outstr = strdup("");
        if (*outstr == NULL) {
            return -1;
        }
        return 0;
    }

    outlen = inlen / 3 * 4 + 4;
    out = malloc(outlen * sizeof(char));
    if (out == NULL) {
        return -1;
    }
    s = out;

    for (i = 0; i < inlen; i += 3) {
        a = in[i];
        b = i + 1 < inlen ? in[i + 1] : 0;
        c = i + 2 < inlen ? in[i + 2] : 0;

        *s++ = b64alpha[a >> 2];
        *s++ = b64alpha[((a & 3) << 4) | (b >> 4)];

        if (i + 1 >= inlen) {
            *s++ = B64PAD;
        } else {
            *s++ = b64alpha[((b & 15) << 2) | (c >> 6)];
        }

        if (i + 2 >= inlen) {
            *s++ = B64PAD;
        } else {
            *s++ = b64alpha[c & 63];
        }
    }

    /* null terminate */
    assert(s != NULL);
    *s = '\0';

    *outstr = out;

    return 0;
}
