/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 * Copyright (c) 2003-2005 LinuxMagic Inc. All Rights Reserved.
 *
 * This file contains Original Code and/or Modifications of Original Code as
 * defined in and that are subject to the Free Source Code License Version
 * 1.0 (the 'License'). You may not use this file except in compliance with
 * the License. Please obtain a copy of the License at:
 *
 * http://www.linuxmagic.com/opensource/licensing/FSCL.txt
 *
 * and read it before using this file.
 *
 * The Original Code and all software distributed under the License are
 * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESS OR IMPLIED, AND LINUXMAGIC HEREBY DISCLAIMS ALL SUCH WARRANTIES,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT. Please see
 * the License for the specific language governing rights and limitations
 * under the License."
 *
 * Please read the terms of this license carefully. By using or downloading
 * this software or file, you are accepting and agreeing to the terms of this
 * license with LinuxMagic Inc. If you are agreeing to this license on behalf
 * of a company, you represent that you are authorized to bind the company to
 * such a license. If you do not meet this criterion or you do not agree to
 * any of the terms of this license, do NOT download, distribute, use or alter
 * this software or file in any way.
 *
 * Author(s): Josh Wilsdon <josh@wizard.ca>
 *
 * CVS Id: $Header: /web/programmers/cvs/magicmail-backend/common/header_parse.c,v 1.4 2005/12/19 21:13:49 josh Exp $
 *
 * DO NOT MODIFY WITHOUT CONSULTING THE LICENSE
 */

#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "header_parse.h"

header_t **readHeaders(FILE * file)
{
    int idx, i, ch, done, count, eol;
    header_t **headers, **tmp;

    if (file == NULL) {
        return (NULL);
    }

    headers = NULL;
    count = idx = done = 0;
    while ((!done) && (count < 1024)) {
        count++;
        tmp = realloc(headers, (sizeof(header_t *) * count));
        if (tmp == NULL) {
            /* perror("realloc()"); */
            return (NULL);
        }
        headers = tmp;

        headers[idx] = malloc(sizeof(header_t));
        if (headers[idx] == NULL) {
            /* perror("malloc()"); */
            return (NULL);
        }
        headers[idx]->data[0] = '\0';
        headers[idx]->length = 0;

        i = eol = 0;
        while ((i < (1024 - 1))
               && ((headers[idx]->data[i] = fgetc(file)) != EOF)) {
            if (headers[idx]->data[i] == '\n') {
                eol = 1;
                if ((i > 0) && (headers[idx]->data[i - 1] == '\r')) {
                    i--;
                }
                if ((ch = fgetc(file)) != EOF) {
                    if ((ch == ' ') || (ch == '\t')) {
                        headers[idx]->data[i] = ch;
                    } else {
                        ungetc(ch, file);
                        break;
                    }
                } else {
                    break;
                }
            }
            i++;
        }
        headers[idx]->data[i] = '\0';

        if (eol == 0) {
            /* consume any remaining portions of a line, since the line is too long */
            while (((ch = fgetc(file)) != EOF) && (ch != '\n')) {
                ;               /* do nothing */
            }
        }

        headers[idx]->length = strlen(headers[idx]->data);
        if (headers[idx]->length == 0) {
            done = 1;
        }
        idx++;
    }

    return (headers);
}

int parseFrom(char *line, char outuser[], char outdomain[])
{
    char *start, *end, *at, *ptr;

    if ((line == NULL) || (outuser == NULL) || (outdomain == NULL)) {
        /* bad data */
        return (1);
    }

    /* skip past : which marks end of header marker */
    ptr = strchr(line, ':');
    if (ptr == NULL) {
        /* bad data */
        return (1);
    }
    ptr++;

    /* skip past any leading spaces */
    while (isspace(*ptr)) {
        ptr++;
    }

    /* 
     * XXX: this will not allow @ character in comments or quotes 
     *      before the @ in the address.  This will fail on some
     *      valid addresses.  
     */
    at = strchr(ptr, '@');
    if (at == NULL) {
        /* bad data */
        return (1);
    }

    start = end = NULL;
    ptr = (at - 1);
    end = at;
    if ((at > line) && (*(at - 1) == '"')) {
        ptr--;
        end--;
        /* XXX: this will fail on From: \\"josh"@wizard.ca */
        while (ptr > line) {
            if ((*ptr == '"') && (*(ptr - 1) != '\'')) {
                start = (ptr + 1);
                break;
            } else {
                ptr--;
            }
        }
    } else {
        while (ptr > line) {
            /* all chars from "atext" RFC2822 3.2.4 */
            if (isalnum(*ptr) || (*ptr == '!') || (*ptr == '#') || (*ptr == '$')
                || (*ptr == '%')
                || (*ptr == '&') || (*ptr == '\'') || (*ptr == '*')
                || (*ptr == '+')
                || (*ptr == '-') || (*ptr == '/') || (*ptr == '=')
                || (*ptr == '?')
                || (*ptr == '^') || (*ptr == '_') || (*ptr == '`')
                || (*ptr == '{')
                || (*ptr == '|') || (*ptr == '}') || (*ptr == '~')
                || (*ptr == '.')) {
                ptr--;
            } else {
                start = (ptr + 1);
                break;
            }
        }
    }

    if (start == NULL) {
        /* printf("WARN: start not found\n"); */
        return (1);
    }

    if ((end - start) > 0 && (end - start) < 100 /* MAX_NAME_LEN */ ) {
        strncpy(outuser, start, (end - start));
        outuser[(end - start)] = '\0';
    } else {
        /* bad data */
        return (1);
    }

    start = end = NULL;
    ptr = (at + 1);
    start = ptr;
    while (*ptr != '\0') {
        if (isalnum(*ptr) || (*ptr == '!') || (*ptr == '#') || (*ptr == '$')
            || (*ptr == '%')
            || (*ptr == '&') || (*ptr == '\'') || (*ptr == '*') || (*ptr == '+')
            || (*ptr == '-') || (*ptr == '/') || (*ptr == '=') || (*ptr == '?')
            || (*ptr == '^') || (*ptr == '_') || (*ptr == '`') || (*ptr == '{')
            || (*ptr == '|') || (*ptr == '}') || (*ptr == '~') || (*ptr == '.')) {
            ptr++;
        } else {
            end = ptr;
            break;
        }
    }
    if (*ptr == '\0') {
        end = ptr;
    }

    if (end == NULL) {
        /* printf("WARN: end not found\n"); */
        return (1);
    }

    if ((end - start) > 0 && (end - start) < 100 /* MAX_DOMAIN_LEN */ ) {
        strncpy(outdomain, start, (end - start));
        outdomain[(end - start)] = '\0';
    } else {
        /* bad data */
        return (1);
    }

    return (0);
}
