/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 * Copyright (c) 2004-2005 LinuxMagic Inc. All Rights Reserved.
 *
 * This file contains Original Code and/or Modifications of Original Code as
 * defined in and that are subject to the Free Source Code License Version
 * 1.0 (the 'License'). You may not use this file except in compliance with
 * the License. Please obtain a copy of the License at:
 *
 * http://www.linuxmagic.com/opensource/licensing/FSCL.txt
 *
 * and read it before using this file.
 *
 * The Original Code and all software distributed under the License are
 * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESS OR IMPLIED, AND LINUXMAGIC HEREBY DISCLAIMS ALL SUCH WARRANTIES,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT. Please see
 * the License for the specific language governing rights and limitations
 * under the License."
 *
 * Please read the terms of this license carefully. By using or downloading
 * this software or file, you are accepting and agreeing to the terms of this
 * license with LinuxMagic Inc. If you are agreeing to this license on behalf
 * of a company, you represent that you are authorized to bind the company to
 * such a license. If you do not meet this criterion or you do not agree to
 * any of the terms of this license, do NOT download, distribute, use or alter
 * this software or file in any way.
 *
 * Author(s): Josh Wilsdon <josh@wizard.ca>
 *
 * CVS Id: $Header: /web/programmers/cvs/magicmail-backend/common/isdsl.c,v 1.7 2005/12/19 21:13:49 josh Exp $
 *
 * DO NOT MODIFY WITHOUT CONSULTING THE LICENSE
 */

#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <syslog.h>
#include <regex.h>
#include "isdsl.h"
#include <common/controls.h>

#define LINE_LEN 1024

extern config_t config;
char default_dslregex[] = "^[^.0-9]*[0-9]{1,3}-[0-9]{1,3}-[0-9]{1,3}-[0-9]{1,3}[^0-9]*.";

/* int main()
{
    // char string[] = "200-204-48-99.dial-up.telesp.net.br";
    char string[] = "www.wizard.ca";
    printf("isDSL(%s): %d\n", string, isDSL(string));
}
*/

int isDSL(char *address) 
{
    regex_t preg;
    char regerrbuf[80];
    int ret;

    if (address == NULL) {
        return (0);
    }

    /* first check the list of regex's in the system */

    ret = checkDSLRegexList(address);
    if (ret >= 0) {
        return (ret);
    }

    /* if we got here, checkDSLRegexList() failed */

    ret = regcomp(&preg, default_dslregex, REG_EXTENDED | REG_NOSUB);
     
    if (ret != 0) {
        /* problem with regular expression */
        regerror(ret, &preg, regerrbuf, sizeof(regerrbuf));
        syslog(LOG_INFO, "Warning: isDSL(): regcomp(): %s\n", regerrbuf);
    } else {
        /* successfully compiled */
        if (regexec(&preg, address, 0, NULL, 0) == 0) {
            /* match */
            regfree(&preg);

            return (1);
        } else {
            /* no match */
        }
    }
    regfree(&preg);

    return (0); /* does not match, or error */
}




int checkDSLRegexList(char *hostname)
{
    regex_t preg;
    int lineno, i, len, ret, returnval;
    char line[LINE_LEN], regerrbuf[80];
    FILE *fp;

    if (config.dynamic_dns_regex_filename == NULL) {
        syslog(LOG_INFO, "Warning: checkDSLRegexList(): filename not set");
        return (-1);
    }

    fp = fopen(config.dynamic_dns_regex_filename, "r");
    if (fp == NULL) {
        syslog(LOG_INFO, "Warning: checkDSLRegexList(): unable to open file");
        return (-1);
    }

    returnval = lineno = 0;
    while (fgets(line, LINE_LEN, fp) != NULL) {
        lineno++;
        len = strlen(line);
        if (line[len - 1] == '\n') {
            line[--len] = '\0';
        }

        if (len > 0) {
            /* Ignore space at the beginning of line when checking for comments */
            i = 0;
            while (isspace(line[i])) {
                i++;
            }

            if (line[i] != '#') {
                ret = regcomp(&preg, line, REG_EXTENDED | REG_NOSUB);
                if (ret != 0) {
                    /* problem with regular expression */
                    regerror(ret, &preg, regerrbuf, sizeof(regerrbuf));
                    syslog(LOG_INFO, "Warning: checkDSLRegexList(): regcomp(line %d): %s",  lineno, regerrbuf);

                    /* set this so that if no regex matches, we return an error */
                    returnval = -1;
                } else {
                    /* successfully compiled */
                    if (regexec(&preg, hostname, 0, NULL, 0) == 0) {
                        /* match */
                        regfree(&preg);
                        syslog(LOG_DEBUG, "checkDSLRegexList(line %d): hit [%s][%s]", lineno, hostname, line);
                        return (1);
                    } else {
                        /* no match */
                    }
                }
                regfree(&preg);
            }
        }
    }

    /* will be 0 if there were no errors and no matches, or -1 if there was an error */
    return (returnval);
}
