/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 * Copyright (c) 2004-2005 LinuxMagic Inc. All Rights Reserved.
 *
 * This file contains Original Code and/or Modifications of Original Code as
 * defined in and that are subject to the Free Source Code License Version
 * 1.0 (the 'License'). You may not use this file except in compliance with
 * the License. Please obtain a copy of the License at:
 *
 * http://www.linuxmagic.com/opensource/licensing/
 *
 * and read it before using this file.
 *
 * The Original Code and all software distributed under the License are
 * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESS OR IMPLIED, AND LINUXMAGIC HEREBY DISCLAIMS ALL SUCH WARRANTIES,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT. Please see
 * the License for the specific language governing rights and limitations
 * under the License."
 *
 * Please read the terms of this license carefully. By using or downloading
 * this software or file, you are accepting and agreeing to the terms of this
 * license with LinuxMagic Inc. If you are agreeing to this license on behalf
 * of a company, you represent that you are authorized to bind the company to
 * such a license. If you do not meet this criterion or you do not agree to
 * any of the terms of this license, do NOT download, distribute, use or alter
 * this software or file in any way.
 *
 * Author(s): Josh Wilsdon <josh@wizard.ca>
 *
 * CVS Id: $Header: /web/programmers/cvs/magicmail-backend/common/load_rules_dbfile.c,v 1.7 2005/12/22 00:50:41 josh Exp $
 *
 * DO NOT MODIFY WITHOUT CONSULTING THE LICENSE
 */

#include <assert.h>
#include <ctype.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <syslog.h>
#include <string.h>
#include <unistd.h>

/* for waitpid */
#include <sys/types.h>
#include <sys/wait.h>

/* DBFile includes */
#include <db.h>

/* local includes */
#include "../common/spam_rules.h"

/* headers */
#include "spam_rules.h"
#include "load_rules_dbfile.h"

#include "controls.h"

extern config_t config;

/** DBFILE */

int getSpamRulesDBF(DB * dbp, char *id, char keychar, unsigned int *dbdatalen, char **dbdata)
{
    unsigned int len;

    if ((id == NULL) || ((len = (unsigned int) strlen(id)) == 0)) {
        return (-1);
    }

    return (getKeyMangledData(dbp, id, keychar, dbdatalen, dbdata));
}


/*      
 * RETURN VALUE
 *
 *  0   success (email_id will be populated)
 *  1   failed, unable to do check
 *  2   failed, internal error (out of memory, etc.)
 *  3   no such email address, or spam disallowed
 *
 */         
int getEmailIdDBFile(char *email, char **email_id)
{
    DB *dbp;
    char *tmp, *data;
    int len, ret;

    dbp = openDBFile(config.user_info_dbfile);
    if (dbp == NULL) {
        return (1);
    }

    /* need to append 'S' to string, so use getKeyMangledData() */
    ret = getKeyMangledData(dbp, email, 'S', &len, &data);

    switch (ret) {
        case -1:
            /* error */
            return (1);
        case 0:
            /* successfully found */
            tmp = strdup(data);
            closeDBFile(dbp);
            if (tmp == NULL) {
                return (2);
            }
            *email_id = tmp;
            return (0);
        case 1:
            /* key not found */
            return (3);

    }

    return (2);
}

int getSpamRulesDBFile(spam_rule_t *rules, char *id, char keychar)
{
    DB *dbp;
    char *data;
    unsigned int len;
    int ret;

    if (id != NULL) {
        dbp = openDBFile(config.spam_rule_dbfile);
        if (dbp == NULL) {
            return (-1);
        }

        /*
         * keychar = R for spam rules       (id = email_id)
         * keychar = L for spam lists       (id = email_id)
         * keychar = T for template rules   (id = template_id)
         *
         * getSpamRules() returns 1 if not found and -1 on error 
         *
         */
        ret = getSpamRulesDBF(dbp, id, keychar, &len, &data);
        if (ret != 0) {
            if (ret == 1) {
                /*
                 * no lists found, but we still need to call loadStringLists so
                 * that the booleans get removed.
                 *
                 */
                loadStringLists(rules, NULL, 0);
            }
            closeDBFile(dbp);
            return (ret);
        }

        /* spam rules are in data */
        if (keychar == 'L') {
            loadStringLists(rules, data, len);
        } else {
            loadStringRules(rules, data, len, NULL);
        }

        closeDBFile(dbp);

        return (0);
    } else {
        syslog(LOG_DEBUG, "load_spam_rules(): did not find id in database");
    }

    return (-1);
}
