/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 * Copyright (c) 2004-2005 LinuxMagic Inc. All Rights Reserved.
 *
 * This file contains Original Code and/or Modifications of Original Code as
 * defined in and that are subject to the Free Source Code License Version
 * 1.0 (the 'License'). You may not use this file except in compliance with
 * the License. Please obtain a copy of the License at:
 *
 * http://www.linuxmagic.com/opensource/licensing/
 *
 * and read it before using this file.
 *
 * The Original Code and all software distributed under the License are
 * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESS OR IMPLIED, AND LINUXMAGIC HEREBY DISCLAIMS ALL SUCH WARRANTIES,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT. Please see
 * the License for the specific language governing rights and limitations
 * under the License."
 *
 * Please read the terms of this license carefully. By using or downloading
 * this software or file, you are accepting and agreeing to the terms of this
 * license with LinuxMagic Inc. If you are agreeing to this license on behalf
 * of a company, you represent that you are authorized to bind the company to
 * such a license. If you do not meet this criterion or you do not agree to
 * any of the terms of this license, do NOT download, distribute, use or alter
 * this software or file in any way.
 *
 * Author(s): Josh Wilsdon <josh@wizard.ca>
 *
 * CVS Id: $Header: /web/programmers/cvs/magicmail-backend/common/load_rules_ext.c,v 1.6 2006/01/02 21:58:05 josh Exp $
 *
 * DO NOT MODIFY WITHOUT CONSULTING THE LICENSE
 */

#include <assert.h>
#include <ctype.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <syslog.h>
#include <string.h>
#include <unistd.h>

/* for waitpid */
#include <sys/types.h>
#include <sys/wait.h>

/* local includes */
#include <common/controls.h>
#include <common/spam_rules.h>
#include <common/load_controls.h>

extern config_t config;

/*      
 * RETURN VALUE
 * 
 *  0   success (email_id will be populated)
 *  1   failed, unable to do check
 *  2   failed, internal error (out of memory, etc.)
 *  3   no such email address, or spam disallowed
 *
 */
int getEmailIdExt(char *email, char **email_id)
{
    char *tmp;

    tmp = strdup(email);
    if (tmp == NULL) {
        return (2);
    }
    *email_id = tmp;

    return (0);
}

int getSpamRulesExt(spam_rule_t *rules, char *email_id)
{
    int srpipe[2], status, len;
    pid_t pid, child;
    char *buffer;

    assert(email_id != NULL);

    if (pipe(srpipe) != 0) {
        syslog(LOG_WARNING, "getSpamRulesExt(): pipe(): %s", strerror(errno));
        return (1);
    }

    switch (pid = fork()) {
        case 0:
            /* child */

            /* TODO move to execSpamdirProg(int child_pipe[]) */
            /* replace stdout with our pipe */
            close(srpipe[0]);
            dup2(srpipe[1], 1);

            /* TODO clean email_id */

            /* run spamdir program */
            execlp(config.ext_spam_rule_prog, config.ext_spam_rule_prog, email_id, NULL);

            /* should never get here, unless spamdir won't run */
            syslog(LOG_ERR, "getSpamRulesExt(): exec(): %s\n", strerror(errno));

            _exit(0);
        case -1:
            /* error */
    
            syslog(LOG_ERR, "getSpamRulesExt(): fork(): %s\n", strerror(errno));

            /* load rules failed */
            return (1);
        default:
            /* parent */

            /* TODO move to manageSpamdirChild(pid, int child_pipe[]) */
            assert(pid > 0);

            /* close our copy of the read pipe, so we get EOF when client is done */
            close(srpipe[1]);

            /* XXX this is not robust */
            len = readString(srpipe[0], &buffer);
            newlineToNull(buffer, len); 
            loadStringRules(rules, buffer, len, NULL);
            
            /* wait for child to return */
            child = waitpid(pid, &status, 0);

            if (WIFEXITED(status)) {
                /* this is the return value of ./spamdir */
                if (WEXITSTATUS(status) == 1) {
                    /* failure */ 
                } else {
                    /* success */
                    return (0);
                }
            } else {
                /* child did not exit properly */
                syslog(LOG_ERR, "getSpamRulesExt(): child exited improperly or was killed"); 
            }
            break;
    }

    return (0);
}

