/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 * Copyright (c) 2004-2005 LinuxMagic Inc. All Rights Reserved.
 *
 * This file contains Original Code and/or Modifications of Original Code as
 * defined in and that are subject to the Free Source Code License Version
 * 1.0 (the 'License'). You may not use this file except in compliance with
 * the License. Please obtain a copy of the License at:
 *
 * http://www.linuxmagic.com/opensource/licensing/
 *
 * and read it before using this file.
 *
 * The Original Code and all software distributed under the License are
 * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESS OR IMPLIED, AND LINUXMAGIC HEREBY DISCLAIMS ALL SUCH WARRANTIES,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT. Please see
 * the License for the specific language governing rights and limitations
 * under the License."
 *
 * Please read the terms of this license carefully. By using or downloading
 * this software or file, you are accepting and agreeing to the terms of this
 * license with LinuxMagic Inc. If you are agreeing to this license on behalf
 * of a company, you represent that you are authorized to bind the company to
 * such a license. If you do not meet this criterion or you do not agree to
 * any of the terms of this license, do NOT download, distribute, use or alter
 * this software or file in any way.
 *
 * Author(s): Josh Wilsdon <josh@wizard.ca>
 *
 * CVS Id: $Header: /web/programmers/cvs/magicmail-backend/common/spam_rules.c,v 1.11 2006/01/03 00:41:56 josh Exp $
 *
 * DO NOT MODIFY WITHOUT CONSULTING THE LICENSE
 */


#include <assert.h>
#include <ctype.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <syslog.h>
#include <math.h>
#include <unistd.h>
#include <sys/types.h>
#include <regex.h>

/* local includes */
#include <common/controls.h>
#include "spam_rules.h"

#ifdef USE_LM_DATABASE
#include "postgresql.h"
#endif

extern config_t config;


/** UTILS */

void cleanupBackend(enum backend_type type)
{
    switch (type) {
#ifdef USE_LM_DATABASE
        case DATABASE:
            disconnectDB();
            return;
#endif
        default:
            /* DO NOTHING */
            return;
    }
}


void printRule(char *name)
{
    char tabs[10];
    int count, i, strwidth, width = 32;

    bzero(tabs, sizeof(tabs));
    width = (width / 8) * 8;
    strwidth = strlen(name);

    count = ((width - strwidth) / 8);
    if (((width - strwidth) % 8)) {
         count++;
    }
    for(i=0; i< count; i++) {
        tabs[i] = '\t';
    }
    tabs[i] = '\0';

    printf("%s%s", name, tabs);

    return;
}


void printSpamRules(spam_rule_t *rules)
{
    spam_rule_t *ptr;
    spam_rule_entry_t *value;

    /* printf("----user----\n"); */
    ptr = rules;
    while (ptr->key != NULL) {
        if (ptr->entries > 0) {
            printRule(ptr->key);
            // printf("%s \t%d: ", ptr->key, ptr->entries);
            value = ptr->first;
            while ((value != NULL) && (value->data != NULL)) {
                if (ptr->type == RULE_DOUBLE) {
                    printf("[%f]", (double) *((double *) value->data));
                } else {
                    printf("[%s]", (char *) value->data);
                }
                value = value->next;
            }
            printf("\n");
        }
        ptr++;
    }
}

int isNumeric(char *value)
{
    char *ptr;

    ptr = value;

    /* the first character must be a number */
    if (!isdigit(*ptr)) {
        return (0);
    }

    while (*ptr != '\0') {
        if (!isdigit(*ptr)) {
            return (0);
        }
        ptr++;
    }

    return (1);
}

void newlineToNull(char *str, int str_len)
{
    char *ptr;

    /* replace all '\n's with '\0's */
    ptr = str;
    while ((ptr - str) < str_len) {
        if (*ptr == '\n') {
            *ptr = '\0'; 
        }
        ptr++; 
    }

    return;
}

/* TODO this sucks */
int splitUserDomain(char *email, char **user, char **domain)
{
    char *tmp;

    assert(email != NULL);

    tmp = strdup(email);
    if (tmp == NULL) {
        /* out of memory */
        return (1);
    }

    *user = splitUser(tmp);
    *domain = splitDomain(tmp);
    free(tmp);

    return (0);
}

char *splitUser(char *email)
{
    char *ptr, *out;

    out = NULL;
    ptr = strrchr(email, '@');
    if (ptr != NULL) {
        *ptr = '\0';
        out = strdup(email);
        *ptr = '@';
    } else {
        out = strdup(email);
    }

    return (out);
}

char *splitDomain(char *email)
{
    char *ptr, *out;

    out = NULL;
    ptr = strrchr(email, '@');
    if (ptr != NULL)  {
        out = strdup(ptr+1);
    }

    return (out);
}


/* XXX this will allocate memory until it runs out of either memory or data */
int readString(int fd, char **str)
{
    int bytes, total_bytes, avail_bytes, new_bytes;
    char *buffer, *ptr, *tmp;

    // total_bytes = 1024;     /* start with 1k */
    total_bytes = 10;     /* XXX TODO start with 1k, (NOTE: must start at least 10) */

    buffer = malloc(sizeof(char) * total_bytes);
    if (buffer == NULL) {
        /* XXX allocation problem */
        return (-1);
    }
    avail_bytes = total_bytes;

    // printf("DEBUG: total(%d) avail(%d)\n", total_bytes, avail_bytes);
    ptr = buffer;
    while ((bytes = read(fd, ptr, avail_bytes)) > 0) {
        avail_bytes -= bytes;
        ptr += bytes;

        // printf("DEBUG: read(%d) total(%d) avail(%d)\n", bytes, total_bytes, avail_bytes);

        if (avail_bytes < (total_bytes / 10)) {
            /* less than 10% available */
            new_bytes = (total_bytes * 2);
            // printf("DEBUG: resizing to %d\n", new_bytes);
            tmp = realloc(buffer, new_bytes);
            if (tmp != NULL) {
                // printf("DEBUG: ptr [%p] buffer [%p] tmp [%p]\n", ptr, buffer, tmp);
                ptr = tmp + (ptr - buffer);
                buffer = tmp;
                // printf("DEBUG: ptr [%p] buffer [%p] tmp [%p]\n", ptr, buffer, tmp);
                avail_bytes = avail_bytes + (new_bytes - total_bytes);
                total_bytes = new_bytes;
            } else {
                free(buffer);
                return (-1);
            }
        }
    }

    // tmp = buffer;
    // printf("data(%d)[", (total_bytes - avail_bytes));
    // while(tmp < ptr) {
        // printf("%c", *tmp++);
    // }
    // printf("]\n");

    *str = buffer;
    return (total_bytes - avail_bytes);
}



/** GENERIC */


/* 
 * RETURN VALUES
 *
 * -1   error
 *  0   loaded successfully
 *  1   no spam rules
 *
 */
int loadSpamRules(spam_rule_t *rules, char *email)
{
    int ret = -1;
    int tried = 0;

#ifdef USE_LM_DBFILE
    if (config.use_dbfile == 1) {
        ret = loadSpamRulesHelper(rules, email, DBFILE);
        tried = 1;
        switch (ret) {
            case 0:     /* success */
                return (0);
            case 1:     /* no rules */
            case 2:     /* disallowed */
                return (1);
            default:
                syslog(LOG_WARNING, "Failed to load spam rules from DBFILE");
                /* FALLTHROUGH */
        }
    }
#endif

#ifdef USE_LM_DATABASE
    if ((config.use_database == 1) || (config.fallback_db == 1)) {
        cleanSpamRules(rules);
        ret = loadSpamRulesHelper(rules, email, DATABASE);
        tried = 1;
        switch (ret) {
            case 0:     /* success */
                return (0);
            case 1:     /* no rules */
            case 2:     /* disallowed */
                return (1);
            default:
                syslog(LOG_WARNING, "Failed to load spam rules from DATABASE");
                /* FALLTHROUGH */
        }
    }
#endif

    if (config.ext_spam_rule_prog != NULL) {
        cleanSpamRules(rules);
        ret = loadSpamRulesHelper(rules, email, EXT_PROG);
        tried = 1;
        switch (ret) {
            case 0:     /* success */
                return (0);
            case 1:     /* no rules */
            case 2:     /* disallowed */
                return (1);
            default:
                syslog(LOG_WARNING, "Failed to load spam rules from EXTPROG");
                /* FALLTHROUGH */
        }
    }

    if (tried == 0) {
        return (1);  /* the system is not configured for spam rules */
    }

    return (-1);
}

/*
 * RETURN VALUES
 * 
 *  0   loaded successfully
 *  1   no rules for user
 *  2   disallowed
 *  ---- above here and email_id != 0, we will syslog a "Spam checking:" line
 *  3   unable to check
 *  4   internal error
 *
 */
int loadSpamRulesHelper(spam_rule_t *rules, char *email, enum backend_type type)
{   
    char *email_id, *template_id;
    int global = 0, ret;

    syslog(LOG_DEBUG, "loading rules for [%s] using [%d]", email, type);

    if (strcmp(email, "0") == 0) {
        /* special case for global rules */
        email_id = strdup(email);
        global = 1;
    } else {
        ret = getEmailId(email, &email_id, type);

        switch (ret) {
            case 0:
                /* success */
                break;
            case 1:
                /* failed (unable to check) */
                return (3);
            case 2:
                /* failed (internal error) */
                return (4);
            case 3:
                /* no such email / disallowed */
                syslog(LOG_INFO, "Spam checking: disallowed");
                return (2);
        }
    }

    syslog(LOG_DEBUG, "found email_id = [%s]", email_id);
    ret = getSpamRules(rules, email_id, type);
    switch (ret) {
        case 0:
            /* success */
            break;
        case 1:
            /* no rules for email_id */
            syslog(LOG_INFO, "Spam checking: allowed/disabled");
            return (1);
        default:
            /* internal error */
            return (4);
    }

    template_id = getTemplateID(rules);
    if (template_id != NULL) {
        getTemplateRules(rules, template_id, type);
        setSpamRule(rules, "loaded_template", template_id);
        /* XXX ignore failure for now */
    }

    if (isEnabled(rules, SPAM_RULE_SPAM_CHECK)) {
        if (!global) {
            syslog(LOG_INFO, "Spam checking: allowed/enabled");
        }
    } else {
        if (!global) {
            syslog(LOG_INFO, "Spam checking: allowed/disabled");
        }
    }

    getSpamLists(rules, email_id, type);
    /* XXX ignore failure for now */

    cleanupBackend(type);

    return (0);
}


char * getTemplateID(spam_rule_t *rules)
{
    char *ret;

    if (rules[SPAM_RULE_SPAM_CHECK_LEVEL].entries > 0) {
        assert (rules[SPAM_RULE_SPAM_CHECK_LEVEL].first != NULL);
        if (rules[SPAM_RULE_SPAM_CHECK_LEVEL].first->data != NULL) {
            if (isNumeric(rules[SPAM_RULE_SPAM_CHECK_LEVEL].first->data)) {
                ret = strdup(rules[SPAM_RULE_SPAM_CHECK_LEVEL].first->data);
                if (ret != NULL) {
                    return (ret);
                }
                /* XXX out of memory */
            }
        }
    }

    return (NULL);
}

/* 
 * SUMMARY
 *
 *   TODO
 *
 * LOGIC
 *
 *   loop through each spam rule
 *       if rule.type == RULE_LIST
 *           remove entries for rule
 *           if rule.first.data == "true"   (we use this list) 
 *               load any values for this rule in string
 *
 *   NOTE:   XXX
 *
 *       This is not really efficient because we search the entire string
 *       (via loadStringRules()) for each list rule.  
 *       Possible optimization point.
 *       
 */
int loadStringLists(spam_rule_t *rules, char *str, int str_len)
{
    spam_rule_t *ruleptr;
    int use_rule;

    ruleptr = rules;
    while (ruleptr->key != NULL) {
        if ((ruleptr->entries > 0) && (ruleptr->type == RULE_LIST)) {

            use_rule = 0;
            if ((ruleptr->first != NULL) && (ruleptr->first->data != NULL)) {
                if (strcmp(ruleptr->first->data, "true") == 0) {
                    use_rule = 1;
                }
            }

            /* free the entries attached to this rule */
            freeRuleEntries(ruleptr->first);

            /* finally zero the entries counter, and pointers, since we free()'d
             * all entries */
            ruleptr->entries = 0;
            ruleptr->first = NULL;
            ruleptr->last = NULL;

            if ((str != NULL) && (use_rule == 1)) {
                loadStringRules(rules, str, str_len, ruleptr->key);
            }
        }
        ruleptr++;
    }

    return (0);
}


int loadStringRules(spam_rule_t *rules, char *str, int str_len, char *rule)
{
    char *ptr, *key, *value;

    assert(str != NULL);

    ptr = str;
    while ((ptr - str) < str_len) {
        key = ptr;
        ptr += strlen(key) + 1;
        if ((ptr - str) < str_len) {
            value = ptr;
            ptr += strlen(value) + 1;
            if ((rule == NULL) || ((rule != NULL) && (strcmp(key,rule) == 0))) {
                setSpamRule(rules, key, value);
            } 
        } else {
            syslog(LOG_ERR, "Warning: corrupted spam rules");
            return (1);
        }
    }

    return (0);
}



int addValue(spam_rule_t * rules, char *value)
{
    spam_rule_entry_t *new;
    char *end;

    new = malloc(sizeof(spam_rule_entry_t));
    /* TODO: free this later */

    if (new == NULL) {
 //       syslog(LOG_ERR, "addValue(): out of memory\n");
        exit(111);
    }

    /* start with a fresh entry */
    bzero(new, sizeof(spam_rule_entry_t));

    switch (rules->type) {
        case RULE_DOUBLE:
            if ((new->data = malloc(sizeof(double))) != NULL) {
                *((double *)new->data) = strtod(value, &end);
		if (end == value) {
		    *((double *)new->data) = HUGE_VAL;    
		}
            }
            /* XXX else error */

            break;
        default:

            /* attempt to duplicate the value */
            new->data = strdup(value);
            /* TODO: free this later */

            if (new->data == NULL) {
                exit(111);
            }
            break;
    }

    if (rules->first == NULL) {
        /* this is first (thereby only) rule */

        rules->first = rules->last = new;

    } else {
        /* this is additional rule */

        rules->last->next = new;
        rules->last = new;

    }

    /* added an entry */
    rules->entries++;

    return (0);
}


int setSpamRule(spam_rule_t *rules, char *key, char *value)
{
    assert(rules != NULL);

    while (rules->key != NULL) {
        /* TODO: strncmp() */
        if (strcmp(rules->key, key) == 0) {
            /* match */
            addValue(rules, value);
        }
        rules++;
    }

    return (0);
}

/*
 * free all entries recursively from entry down the chain
 *
 */
void freeRuleEntries(spam_rule_entry_t * entry)
{
    if (entry == NULL) {
        return;
    } else {
        if (entry->data != NULL) {
            free(entry->data);
        }
        freeRuleEntries(entry->next);
        free(entry);
    }
    return;
}

/*
 *  This function cleans out the user data from the spam_rule_t structure
 *  pointed to by "rules".  It will zero the entries counter, and free all
 *  entries attached to each rule.
 *
 */
void cleanSpamRules(spam_rule_t * rules)
{

    while (rules->key != NULL) {
        if (rules->entries > 0) {

            /* free the entries attached to this rule */
            freeRuleEntries(rules->first);

            /* finally zero the entries counter, and pointers, since we free()'d all entries */
            rules->entries = 0;
            rules->first = NULL;
            rules->last = NULL;
        }
        rules++;
    }

    return;
}



/* DISPATCHERS */

/* RETURN VALUES:
 *
 *   1 spam rules do not exist for email_id
 *   0 spam rules loaded successfully
 *  -1 unsupported backend type
 *  -2 internal error
 *
 */
int getSpamRules(spam_rule_t *rules, char *email_id, enum backend_type type)
{
    assert(email_id != NULL);

    switch (type) {
#ifdef USE_LM_DATABASE
        case DATABASE:
            return (getSpamRulesByEmailIdDB(rules, email_id));
#endif
#ifdef USE_LM_DBFILE
        case DBFILE:
            return (getSpamRulesDBFile(rules, email_id, 'R'));
#endif
        case EXT_PROG:
            return (getSpamRulesExt(rules, email_id));
        case NONE:
            return (-1);
        default:
            /* ERROR */
            break;
    } 

    return (-2);
}


/* RETURN VALUES:
 *
 *   1 spam rules do not exist for email_id
 *   0 spam rules loaded successfully
 *  -1 unsupported backend type
 *  -2 internal error
 *
 */
int getSpamLists(spam_rule_t *rules, char *email_id, enum backend_type type)
{
    assert(email_id != NULL);

    rules = rules;

    switch (type) {
#ifdef USE_LM_DATABASE
        case DATABASE:
            return (getSpamListsDB(rules, email_id));
#endif
#ifdef USE_LM_DBFILE
        case DBFILE:
            return (getSpamRulesDBFile(rules, email_id, 'L'));
#endif
        case EXT_PROG:
            /* with EXT_PROG, spam lists are already loaded with rules */
            return (0);
        case NONE:
            return (-1);
        default:
            /* ERROR */
            break;
    } 

    return (-2);
}


int getTemplateRules(spam_rule_t *rules, char *template_id, enum backend_type type)
{
    int retval;
    char *spam_check = NULL;

    assert(template_id != NULL);
    assert(rules != NULL);

    /* save value of "spam_check" rule */
    if (rules[SPAM_RULE_SPAM_CHECK].entries > 0) {
        assert (rules[SPAM_RULE_SPAM_CHECK].first != NULL);
        if (rules[SPAM_RULE_SPAM_CHECK].first->data != NULL) {
            spam_check = strdup(rules[SPAM_RULE_SPAM_CHECK].first->data);
            if (spam_check == NULL) {
                return (1);  /* an error happened in strdup */
            }
        }
    }

    /* clear the user rules, get the template rules */
    cleanSpamRules(rules);

    /* dispatch the proper function */
    switch (type) {
#ifdef USE_LM_DATABASE
        case DATABASE:
            retval = getTemplateRulesDB(rules, template_id);
            break;
#endif
#ifdef USE_LM_DBFILE
        case DBFILE:
            retval = getSpamRulesDBFile(rules, template_id, 'T');
            break;
#endif
        case EXT_PROG:
            /* we currently do not support templates with the ext_prog method */
             retval = 1;
             break;
        case NONE:
            /* FALLTHROUGH */
        default:
            /* ERROR */
            retval = -1;
            break;
    }

    /* if the template does not set the spam_check rule (it shouldn't),
       set spam_check rule to what the user had it set to. */
    if ((rules[SPAM_RULE_SPAM_CHECK].entries == 0) && (spam_check != NULL)) {
        setSpamRule(rules, "spam_check", spam_check);
    }

    if (spam_check != NULL) {
        free(spam_check);   /* allocated by strdup */
    }

    return (retval);    /* the value of getSpamRulesDB() */
}



/*
 * RETURN VALUE
 *
 *  0   success (email_id will be populated)
 *  1   failed, unable to do check
 *  2   failed, internal error (out of memory, etc.)
 *  3   no such email address, or spam disallowed
 *
 */
int getEmailId(char *email, char **email_id, enum backend_type type)
{
    switch (type) {
#ifdef USE_LM_DATABASE
        case DATABASE:
            return (getEmailIdDB(email, email_id));
#endif
#ifdef USE_LM_DBFILE
        case DBFILE:
            return (getEmailIdDBFile(email, email_id));
#endif
        case EXT_PROG:
            return (getEmailIdExt(email, email_id));
        default:
            return (1);
    }
}

unsigned char isEnabled(spam_rule_t *rules, int rule)
{
    if ((rules[rule].type == RULE_BOOL)
        && (rules[rule].entries > 0)
        && (rules[rule].first != NULL)
        && (rules[rule].first->data != NULL)
        && (strncmp(rules[rule].first->data, "true", 4) == 0)) {
        /* enabled */
        return ((unsigned char) 1);
    }

    /* not enabled */
    return ((unsigned char) 0);
}

int disableSpamRule(spam_rule_t *rules, int rule) 
{
    if ((rules[rule].type == RULE_BOOL) && (rules[rule].entries > 0) && (rules[rule].first != NULL)) {
        freeRuleEntries(rules[rule].first);
        rules[rule].entries = 0;
        rules[rule].first = NULL;
        rules[rule].last = NULL;
    }

    addValue((rules + rule), "false");

    return (0);
}


int enableSpamRule(spam_rule_t *rules, int rule) 
{
    if ((rules[rule].type == RULE_BOOL) && (rules[rule].entries > 0) && (rules[rule].first != NULL)) {
        freeRuleEntries(rules[rule].first);
        rules[rule].entries = 0;
        rules[rule].first = NULL;
        rules[rule].last = NULL;
    }

    addValue((rules + rule), "true");

    return (0);
}


/*
 * SUMMARY
 *
 *  This function checks whether the line matches any of a regular expression
 *  list.  (ie subject_whitelist)
 *
 * RETURN VALUE:
 *
 *  score to be added
 *
 */
int checkListRegex(spam_rule_t *rules, int rule, char *line)
{
    return (checkList(rules, rule, line, 0));
}

int checkListStringCmp(spam_rule_t *rules, int rule, char *line)
{
    return (checkList(rules, rule, line, 1));
}

int checkList(spam_rule_t *rules, int rule, char *line, int type)
{
    spam_rule_entry_t *value;
    regex_t preg;
    char regerrbuf[80];
    int ret;

    if (rules[rule].entries <= 0) {
        /* no entries, so don't do this check */
        return (0);
    }

    value = rules[rule].first;
    while (value != NULL) {
        if (value->data == NULL) {
            syslog(LOG_DEBUG, "checkList(): entry had no data.\n");
            return (0);
        }

        if (type == 0) {
            /* regex */
            ret = regcomp(&preg, (char *) value->data /* assumed to be regex */ , REG_ICASE | REG_EXTENDED | REG_NOSUB);
            if (ret != 0) {
                /* problem with regular expression */
                regerror(ret, &preg, regerrbuf, sizeof(regerrbuf));
                syslog(LOG_INFO, "Warning: regcomp(): %s\n", regerrbuf);
            } else {
                /* successfully compiled */
                if (regexec(&preg, line, 0, NULL, 0) == 0) {
                    /* match */
                    regfree(&preg);
                    return (1);
                }
            }
            regfree(&preg);
        } else if (type == 1) {
            /* string compare */
            if (strcmp((char *)value->data, line) == 0) {
                return (1);
            }
        }

        value = value->next;
    }

    return (0);
}


int getScore(spam_rule_t *rules, int rule)
{
    return (rules[rule].score);
}


