/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 * Copyright (c) 2003-2005 LinuxMagic Inc. All Rights Reserved.
 *
 * This file contains Original Code and/or Modifications of Original Code as
 * defined in and that are subject to the Free Source Code License Version
 * 1.0 (the 'License'). You may not use this file except in compliance with
 * the License. Please obtain a copy of the License at:
 *
 * http://www.linuxmagic.com/opensource/licensing/FSCL.txt
 *
 * and read it before using this file.
 *
 * The Original Code and all software distributed under the License are
 * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESS OR IMPLIED, AND LINUXMAGIC HEREBY DISCLAIMS ALL SUCH WARRANTIES,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT. Please see
 * the License for the specific language governing rights and limitations
 * under the License."
 *
 * Please read the terms of this license carefully. By using or downloading
 * this software or file, you are accepting and agreeing to the terms of this
 * license with LinuxMagic Inc. If you are agreeing to this license on behalf
 * of a company, you represent that you are authorized to bind the company to
 * such a license. If you do not meet this criterion or you do not agree to
 * any of the terms of this license, do NOT download, distribute, use or alter
 * this software or file in any way.
 *
 * Author(s): Josh Wilsdon <josh@wizard.ca>
 *
 * CVS Id: $Header: /web/programmers/cvs/magicmail-backend/common/spam_rules.h,v 1.6 2006/01/03 00:41:56 josh Exp $
 *
 * DO NOT MODIFY WITHOUT CONSULTING THE LICENSE
 */


#ifndef SPAM_RULES_H
#define SPAM_RULES_H

#define RULE_BOOL   1
#define RULE_INT    2
#define RULE_DOUBLE 3
#define RULE_STRING 4 
#define RULE_LIST   5

#include <common/spam_rules.inc>

typedef struct _spam_rule_entry_t
{
    void *data;
    struct _spam_rule_entry_t *next;
} spam_rule_entry_t;

typedef struct
{
    char *key;
    unsigned int entries;   /* number of records */
    unsigned int type;      /* int, bool, string, list */
    int score;              /* spam weighting */
    spam_rule_entry_t *last;
    spam_rule_entry_t *first;
} spam_rule_t;


/*
 *    PUBLIC: this data is available to calling modules
 */


/* PUBLIC TYPES */
enum backend_type { NONE, DATABASE, DBFILE, EXT_PROG };


/* PUBLIC FUNCTIONS */
int getSpamRules(spam_rule_t *rules, char *email_id, enum backend_type type);
int getSpamLists(spam_rule_t *rules, char *email_id, enum backend_type type);
int getEmailId(char *email, char **email_id, enum backend_type type);
int getTemplateRules(spam_rule_t *rules, char *template_id, enum backend_type type);
void printSpamRules(spam_rule_t *rules);
int setSpamRule(spam_rule_t *rules, char *key, char *value);
char * getTemplateID(spam_rule_t *rules);
void cleanupBackend(enum backend_type type);
int loadSpamRules(spam_rule_t *rules, char *email);
unsigned char isEnabled(spam_rule_t *rules, int rule);
int disableSpamRule(spam_rule_t *rules, int rule);
int enableSpamRule(spam_rule_t *rules, int rule);
void cleanSpamRules(spam_rule_t * rules);
int checkListRegex(spam_rule_t *rules, int rule, char *line);
int checkListStringCmp(spam_rule_t *rules, int rule, char *line);
int getScore(spam_rule_t *rules, int rule);

/*
 *    PRIVATE: nothing beyond this should be used by outside modules 
 */


/* exported functions (exported to spam_rules) for DB module */
int getSpamRulesByEmailIdDB(spam_rule_t *rules, char *email_id);
int getEmailIdDB(char *email, char **email_id);
int getTemplateRulesDB(spam_rule_t *rules, char *template_id);
int getSpamListsDB(spam_rule_t *rules, char *email_id);
void cleanupDB(void);

/* exported functions (exported to spam_rules) for DBFile module */
int getEmailIdDBFile(char *email, char **email_id);
int getSpamRulesDBFile(spam_rule_t *rules, char *id, char keychar);

/* exported functions (exported to spam_rules) for EXT module */
int getEmailIdExt(char *email, char **email_id);
int getSpamRulesExt(spam_rule_t *rules, char *email_id);

/* private internal functions */
int loadSpamRulesHelper(spam_rule_t *rules, char *email, enum backend_type type);
void freeRuleEntries(spam_rule_entry_t * entry);
int splitUserDomain(char *email, char **user, char **domain);
int setSpamRule(spam_rule_t *rules, char *key, char *value);
int loadStringLists(spam_rule_t *rules, char *str, int str_len);
int loadStringRules(spam_rule_t *rules, char *str, int str_len, char *rule);
void newlineToNull(char *str, int str_len);
int readString(int fd, char **str);
char *splitUser(char *email);
char *splitDomain(char *email);
int addValue(spam_rule_t * rules, char *value);
void printRule(char *name);
int checkList(spam_rule_t *rules, int rule, char *line, int type);


#endif
