/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 * Copyright (c) 2003-2005 LinuxMagic Inc. All Rights Reserved.
 *
 * This file contains Original Code and/or Modifications of Original Code as
 * defined in and that are subject to the Free Source Code License Version
 * 1.0 (the 'License'). You may not use this file except in compliance with
 * the License. Please obtain a copy of the License at:
 *
 * http://www.linuxmagic.com/opensource/licensing/FSCL.txt
 *
 * and read it before using this file.
 *
 * The Original Code and all software distributed under the License are
 * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESS OR IMPLIED, AND LINUXMAGIC HEREBY DISCLAIMS ALL SUCH WARRANTIES,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT. Please see
 * the License for the specific language governing rights and limitations
 * under the License."
 *
 * Please read the terms of this license carefully. By using or downloading
 * this software or file, you are accepting and agreeing to the terms of this
 * license with LinuxMagic Inc. If you are agreeing to this license on behalf
 * of a company, you represent that you are authorized to bind the company to
 * such a license. If you do not meet this criterion or you do not agree to
 * any of the terms of this license, do NOT download, distribute, use or alter
 * this software or file in any way.
 *
 * Author(s): Josh Wilsdon <josh@wizard.ca>
 *
 * Version: $Header: /web/programmers/cvs/magicmail-backend/common/strutil.c,v 1.2 2006/01/10 18:33:29 josh Exp $
 *
 * DO NOT MODIFY WITHOUT CONSULTING THE LICENSE
 *
 */

#include <assert.h>
#include <string.h>
#include <stdlib.h>

#include <common/strutil.h>


/* 
 * SUMMARY
 *
 * This function assumes src is a '\0' terminated string, and that
 * if dest is non-NULL, it also is a '\0' terminated string.  It appends
 * src to dest and allocates memory (and updates destbytes) if dest is
 * not large enough. 
 *
 * RETURN VALUES
 *
 *   0  success
 *  -1  realloc() failed (check errno)
 *
 *
 */
int appendStringSimple(char **dest, size_t *destbytes, char *src)
{
    size_t destlen = 0;
    char *destptr;

    assert(dest != NULL);
    assert(destbytes != NULL);
    assert(src != NULL);

    destptr = *dest;
    if (destptr != NULL) {
        destlen = strlen(destptr); 
    }

    return (appendString(dest, destbytes, &destlen, src, strlen(src)));
}



int appendString(char **dest, size_t *destbytes, size_t *destlen, char *src, size_t srclen)
{
    size_t newsize;
    char *newptr;
    int init = 0;

    assert(dest != NULL);
    assert(destbytes != NULL);
    assert(destlen != NULL);
    assert(src != NULL);
    assert(srclen > 0);

    if ((*dest == NULL) || (*destbytes == 0)) {
        init = 1;
        if ((srclen + 1) < MIN_STRSIZE) {
            newsize = MIN_STRSIZE;
        } else {
            newsize = srclen + 1;
        }
    } else {
        newsize = *destbytes;
    }

    if (*dest != NULL) {
        while ((srclen + *destlen + 1) >= newsize) {
            newsize *= 2;
        }
    }

    newptr = (char *)realloc(*dest, newsize);
    if (newptr == NULL) {
        return (-1);
    }

    if (init == 1) {
        newptr[0] = '\0';
    }

    memcpy((newptr + *destlen), src, srclen);
    memcpy(dest, &newptr, sizeof(dest));
    *destlen += srclen;
    (*dest)[*destlen] = '\0';
    *destbytes = newsize;

    return (0);
}


#ifdef TEST_MAIN
int main()
{
    unsigned int i, j;
    char testdata[] = "hello";
    size_t destlen, destbytes;
    char *dest;

    dest = NULL;
    destlen = destbytes = 0;

    for (i = 0; i < 100; i++) {
        //printf("appendString() = %d\n", appendString(&dest, &destbytes, &destlen, testdata, strlen(testdata) + 1));
        printf("appendStringSimple() = %d\n", appendStringSimple(&dest, &destbytes, testdata));
        printf("string[%d] = [", destlen);
        for (j = 0; j < strlen(dest); j++) {
            if (dest[j] == '\0') {
                printf("\\0");
            } else {
                printf("%c", dest[j]);
            }
        }
        printf("]\n");
    }

    exit (0);
}
#endif
