/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * Copyright (c) 2003-2005 LinuxMagic Inc. All Rights Reserved.
 *
 * This file contains Original Code and/or Modifications of Original Code as
 * defined in and that are subject to the Free Source Code License Version
 * 1.0 (the 'License'). You may not use this file except in compliance with
 * the License. Please obtain a copy of the License at:
 *
 * http://www.linuxmagic.com/opensource/licensing/FSCL.txt
 *
 * and read it before using this file.
 *
 * The Original Code and all software distributed under the License are
 * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESS OR IMPLIED, AND LINUXMAGIC HEREBY DISCLAIMS ALL SUCH WARRANTIES,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT. Please see
 * the License for the specific language governing rights and limitations
 * under the License."
 *
 * Please read the terms of this license carefully. By using or downloading
 * this software or file, you are accepting and agreeing to the terms of this
 * license with LinuxMagic Inc. If you are agreeing to this license on behalf
 * of a company, you represent that you are authorized to bind the company to
 * such a license. If you do not meet this criterion or you do not agree to
 * any of the terms of this license, do NOT download, distribute, use or alter
 * this software or file in any way.
 *
 * Author(s): Josh Wilsdon <josh@wizard.ca>
 *
 * CVS Id: $Header: /web/programmers/cvs/magicmail-backend/common/write_spam_log.c,v 1.9 2005/12/22 00:50:41 josh Exp $
 *
 * DO NOT MODIFY WITHOUT CONSULTING THE LICENSE
 */

/* Standard C includes */
#define _GNU_SOURCE             /* do this so we can use strdup() */
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <syslog.h>
#include <sys/wait.h>
#include <time.h>
#include <fcntl.h>
#include <ctype.h>
#include <errno.h>
#include <sys/stat.h>
#include <signal.h>
#include <string.h>

/* magic-smtpd includes */
#include <magic-smtpd/smtp-limits.h>
#include <common/stopwatch.h>
#include <common/spam_log.h>
#include <common/write_spam_log.h>

/* NEW STYLE */
#include <common/controls.h>
#include <common/load_controls.h>

extern config_t config;
extern spam_log_t spam_log;

/* XXX
 *
 * NOTE: this is a huge ugly mess that should go away as soon as possible.
 *
 */

int writeSpamLogFile(time_t now, char *filename);
char * cleanSpamInfoCSV(char *string);


int writeSpamLog(char *insubject)
{
    time_t now;

    if (spam_log.entries <= 0) {
        syslog(LOG_DEBUG, "writeSpamLog(): no entries to write");
        return (0);
    }

    if ((spam_log.first == NULL) || (spam_log.last == NULL)) {
        syslog(LOG_ERR, "writeSpamLog(): corrupted spam_log structure");
        return (1);
    }

    now = time(NULL);
    if (now == ((time_t) -1)) {
        syslog(LOG_ERR, "writeSpamLog(): time(): %s", strerror(errno)); 
        return (1);
    }

    if (insubject != NULL) {
        spam_log.subject = strdup(insubject);
        if (spam_log.subject == NULL) {
            syslog(LOG_ERR, "writeSpamLog(): out of memory\n");
            return (1);
        }
    } else {
        spam_log.subject = insubject /* NULL */;
    }

#ifdef USE_LM_DATABASE
    if (config.spam_log_db == 1) {
        writeSpamLogDB(now);
    } 
#endif

    if (config.spam_log_file != NULL) {
        writeSpamLogFile(now, config.spam_log_file);
    }

    return (0);
}    


/* 
 * NOTE: this will replace all \r and \n characters with a single space.
 *
 */
char * cleanSpamInfoCSV(char *string)
{
    int i, j, len;
    char *tmp;

    if (string == NULL) {
        return (NULL);
    }

    len = strlen(string);

    /* worst case is a string full of '"' characters */
    tmp = malloc(((len * 2) + 1) * sizeof(char));

    if (tmp == NULL) {
        syslog(LOG_ERR, "cleanSpamInfoDB(): malloc(): %s", strerror(errno));
        return (NULL);
    }

    j = 0;
    for (i = 0; i < len; i++) {
	switch (string[i]) {
	    case '\n':
		    /* FALLTHROUGH */
	    case '\r':
		    tmp[j++] = ' ';
		    break;
	    case '"':
		    /* there is a double quote, escape it */
		    tmp[j++] = '"';
		    /* FALLTHROUGH */
	    default:
		    tmp[j++] = string[i];	
		    break;
	}
    }

    /* terminate */
    tmp[j] = '\0';

    return (tmp); 
}


/*
 * This converts the spam_log entry to CSV format
 *
 * See:
 *
 * 	http://www.ietf.org/internet-drafts/draft-shafranovich-mime-csv-05.txt
 * 	http://en.wikipedia.org/wiki/Comma-separated_values
 *
 *
 */
int writeSpamLogFile(time_t now, char *filename)
{
    int ret = 0;
    char *subject, *sender, *domain, *user, *info = NULL;
    char hostname[1024];
    FILE *fp;
    spam_log_entry_t *ptr;

    subject = cleanSpamInfoCSV(spam_log.subject);
    sender = cleanSpamInfoCSV(spam_log.sender);
    domain = cleanSpamInfoCSV(spam_log.domain);
    user = cleanSpamInfoCSV(spam_log.user);

    if (gethostname(hostname, 1024) != 0) {
        strcpy(hostname, "unknown");
    }

    if ((user == NULL) || (domain == NULL) || (sender == NULL)) {
        /* cleanSpamInfoDB() should have printed a message */ 
        return (1);
    }

    fp = fopen(filename, "a");
    if (fp == NULL) {
        syslog(LOG_ERR, "writeSpamLogFile(): fopen(): %s", strerror(errno));
        return (1);
    }

    ptr = spam_log.first;
    while (ptr != NULL) {
        if (ptr->rule != NULL) {
            info = cleanSpamInfoCSV(ptr->info);
        }

	/* should lock file to ensure exclusivity? */

	fprintf(fp, "\"%lu\",\"%s\",\"%s\",\"%s\",\"%s\",\"%s\",\"%s\",\"%s\",\"%d\",\"%s\",\"%s\"\r\n"
                  , now
                  , hostname
	              , user
                  , domain
                  , sender
                  , (subject == NULL) ? "" : subject
                  , ptr->rule
                  , (info == NULL) ? "" : info
                  , ptr->score
                  , (config.tcpremoteip != NULL) ? config.tcpremoteip : "magic-local"
                  , (ptr->global == 1) ? "Y" : "N");


        if (info != NULL) {
            free(info);
        }

        ptr = ptr->next;
    }

    fclose(fp);

    if (subject != NULL) {
        free(subject);
    }
    if (sender != NULL) {
        free(sender);
    }
    if (domain != NULL) {
        free(domain);
    }
    if (user != NULL) {
        free(user);
    }

    return (ret);
}

