/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 * Copyright (c) 2002-2005 LinuxMagic Inc. All Rights Reserved.
 *
 * This file contains Original Code and/or Modifications of Original Code as
 * defined in and that are subject to the Free Source Code License Version
 * 1.0 (the 'License'). You may not use this file except in compliance with
 * the License. Please obtain a copy of the License at:
 *
 * http://www.linuxmagic.com/opensource/licensing/FSCL.txt
 *
 * and read it before using this file.
 *
 * The Original Code and all software distributed under the License are
 * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESS OR IMPLIED, AND LINUXMAGIC HEREBY DISCLAIMS ALL SUCH WARRANTIES,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT. Please see
 * the License for the specific language governing rights and limitations
 * under the License."
 *
 * Please read the terms of this license carefully. By using or downloading
 * this software or file, you are accepting and agreeing to the terms of this
 * license with LinuxMagic Inc. If you are agreeing to this license on behalf
 * of a company, you represent that you are authorized to bind the company to
 * such a license. If you do not meet this criterion or you do not agree to
 * any of the terms of this license, do NOT download, distribute, use or alter
 * this software or file in any way.
 *
 * Author(s): Burton Samograd <burton@wizard.ca>
 *
 * CVS Id: $Header: /web/programmers/cvs/magicmail-backend/lmlibs/examples/lmconfig-test.c,v 1.3 2005/12/19 21:13:49 josh Exp $
 *
 * DO NOT MODIFY WITHOUT CONSULTING THE LICENSE
 */

#include <stdio.h>

#include <liblm.h>

int main(int argc, char **argv)
{
    lm_string_t controldir;
    lm_config_string_t config_string;
    lm_config_string_t config_string_default;
    lm_config_number_t config_number;
    lm_config_number_t config_number_default;
    lm_config_bool_t config_bool;
    lm_config_bool_t config_bool_default;
    lm_config_string_t string_key;
    lm_config_string_t number_key;
    lm_config_string_t bool_key;

    /* Initialize pointers to null */
    controldir = NULL;
    string_key = number_key = bool_key = NULL;
    config_string = NULL;
    config_number = config_bool = 0;

    LM_LOG_INIT("./lmconfig-test.log");

    /* THis will attempt to setup lmconfig to use control files
       and return an error if it can't */
    LM_STRING_NEW(controldir, "./control/");
    if (LM_CONFIG_CONTROL_SET_PARAM(controldir)) {
        LM_LOG_WARN("lmconfig-test",
                    "Control directory ./control does not exist!");
    }

    /* Allocate the keys to access the configuration variables */
    LM_STRING_NEW(string_key, "configstring");
    LM_STRING_NEW(number_key, "confignumber");
    LM_STRING_NEW(bool_key, "configbool");

    config_string_default = NULL;
    LM_STRING_NEW(config_string_default, "default value");
    config_number_default = -1;
    config_bool_default = 0;

    /* Read the configuration variables from the config */
    LM_CONFIG_READ_STRING(string_key, config_string, config_string_default);
    LM_CONFIG_READ_NUMBER(number_key, config_number, config_number_default);
    LM_CONFIG_READ_BOOL(bool_key, config_bool, config_bool_default);

    /* Read the configuration variables from the config */
    LM_CONFIG_WRITE_STRING(string_key, config_string);
    LM_CONFIG_WRITE_NUMBER(number_key, config_number);
    LM_CONFIG_WRITE_BOOL(bool_key, config_bool);

    /* Close the configuration when done */
    LM_CONFIG_CLOSE;

    /* Free allocated memory */
    LM_STRING_FREE(controldir);
    LM_STRING_FREE(string_key);
    LM_STRING_FREE(number_key);
    LM_STRING_FREE(bool_key);

    /* Test environment config type */
    LM_CONFIG_ENV_SET_PARAM;

    LM_STRING_NEW(string_key, "USER");
    LM_STRING_FREE(config_string_default);
    LM_STRING_NEW(config_string_default, "username unknown");
    LM_CONFIG_READ_STRING(string_key, config_string, config_string_default);

    LM_STRING_FREE(string_key);
    LM_STRING_NEW(string_key, "UNKNOWN");
    LM_STRING_FREE(config_string_default);
    LM_STRING_NEW(config_string_default, "default value");
    LM_CONFIG_READ_STRING(string_key, config_string, config_string_default);

    LM_CONFIG_CLOSE;

    LM_STRING_FREE(string_key);
    LM_STRING_FREE(config_string_default);

    /* Free this when it gets filled in by config */
    LM_STRING_FREE(config_string);

    LM_MEMDUMP(stdout);

    return 0;
}
