/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 * Copyright (c) 2002-2005 LinuxMagic Inc. All Rights Reserved.
 *
 * This file contains Original Code and/or Modifications of Original Code as
 * defined in and that are subject to the Free Source Code License Version
 * 1.0 (the 'License'). You may not use this file except in compliance with
 * the License. Please obtain a copy of the License at:
 *
 * http://www.linuxmagic.com/opensource/licensing/FSCL.txt
 *
 * and read it before using this file.
 *
 * The Original Code and all software distributed under the License are
 * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESS OR IMPLIED, AND LINUXMAGIC HEREBY DISCLAIMS ALL SUCH WARRANTIES,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT. Please see
 * the License for the specific language governing rights and limitations
 * under the License."
 *
 * Please read the terms of this license carefully. By using or downloading
 * this software or file, you are accepting and agreeing to the terms of this
 * license with LinuxMagic Inc. If you are agreeing to this license on behalf
 * of a company, you represent that you are authorized to bind the company to
 * such a license. If you do not meet this criterion or you do not agree to
 * any of the terms of this license, do NOT download, distribute, use or alter
 * this software or file in any way.
 *
 * Author(s): Burton Samograd <burton@wizard.ca>
 *
 * CVS Id: $Header: /web/programmers/cvs/magicmail-backend/lmlibs/examples/lmstring-test.c,v 1.3 2005/12/19 21:13:49 josh Exp $
 *
 * DO NOT MODIFY WITHOUT CONSULTING THE LICENSE
 */

#include <stdio.h>

#include <liblm.h>

int main(int argc, char **argv)
{
    lm_string_t string, string2, string3, string4;
    size_t bytesread;

    string = string2 = string3 = string4 = NULL;

    LM_LOG_INIT("/tmp/garbage");

    LM_STRING_NEW(string, "");
    LM_STRING_NEW(string2, "I am string 2");
    LM_STRING_NEW(string3, "I am string 3");

    LM_MEMSTAT(stdout);

    LM_STRING_CAT(string, string2);
    puts("string: ");
    puts(LM_STRING_BUFFER(string));

    LM_STRING_CAT(string2, string3);
    puts("string2: ");
    puts(LM_STRING_BUFFER(string2));

    LM_STRING_DUP(string4, string3);
    puts("string4: ");
    puts(LM_STRING_BUFFER(string4));

    LM_STRING_FREE(string3);
    LM_STRING_CAT(string4, string2);
    puts("string4: ");
    puts(LM_STRING_BUFFER(string4));

    LM_MEMSTAT(stdout);
    LM_MEMDUMP(stdout);

    LM_STRING_FREE(string4);
    LM_STRING_FREE(string2);

    LM_STRING_FREE(string);
    puts("\nTYPE SOMETHING: ");
    LM_STRING_FGETS(string, stdin, 1024, bytesread);
    puts("ECHO: ");
    puts(LM_STRING_BUFFER(string));

    LM_STRING_FREE(string);
    LM_STRING_NEW(string, "string with whitespace at the end    \t\r\n\n");
    puts("LM_STRING_CHOP::This string has whitespace on the end:");
    printf(LM_STRING_BUFFER(string));
    puts("END");
    LM_STRING_CHOP(string);
    puts("This string has been chopped:");
    printf(LM_STRING_BUFFER(string));
    puts("END");
    puts("This text should be directly below the above line with no spaces");

    LM_STRING_FREE(string);
    LM_STRING_NEW(string, " \r\n \t \t \r \n");
    puts("LM_STRING_CHOP::This string is all whitespace:");
    printf(LM_STRING_BUFFER(string));
    puts("END");
    LM_STRING_CHOP(string);
    puts("This is the whitespace string chopped:");
    printf(LM_STRING_BUFFER(string));
    puts("END");
    puts("There should only be END on the above line");

    LM_STRING_FREE(string);
    LM_STRING_NEW(string,
                  "    \t\r\n\nstring with whitespace at the beginning");
    puts("LM_STRING_CHOP_HEAD::This string has whitespace at the start:");
    puts("START");
    printf(LM_STRING_BUFFER(string));
    puts("END");
    LM_STRING_CHOP_HEAD(string);
    puts("This string has been chopped:");
    puts("START");
    printf(LM_STRING_BUFFER(string));
    puts("END");
    puts("This text should be directly below the above line with no spaces");

    LM_STRING_FREE(string);
    LM_STRING_NEW(string, " \r\n \t \t \r \n");
    puts("LM_STRING_CHOP_HEAD::This string is all whitespace:");
    puts("START");
    printf(LM_STRING_BUFFER(string));
    puts("END");
    LM_STRING_CHOP_HEAD(string);
    puts("This is the whitespace string head chopped:");
    puts("START");
    printf(LM_STRING_BUFFER(string));
    puts("END");
    puts("There should only be START and END at the start of two lines with no space between");

    LM_STRING_FREE(string);

    LM_MEMSTAT(stdout);
    LM_MEMDUMP(stdout);

    return 0;
}
