/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 * Copyright (c) 2002-2005 LinuxMagic Inc. All Rights Reserved.
 *
 * This file contains Original Code and/or Modifications of Original Code as
 * defined in and that are subject to the Free Source Code License Version
 * 1.0 (the 'License'). You may not use this file except in compliance with
 * the License. Please obtain a copy of the License at:
 *
 * http://www.linuxmagic.com/opensource/licensing/FSCL.txt
 *
 * and read it before using this file.
 *
 * The Original Code and all software distributed under the License are
 * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESS OR IMPLIED, AND LINUXMAGIC HEREBY DISCLAIMS ALL SUCH WARRANTIES,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT. Please see
 * the License for the specific language governing rights and limitations
 * under the License."
 *
 * Please read the terms of this license carefully. By using or downloading
 * this software or file, you are accepting and agreeing to the terms of this
 * license with LinuxMagic Inc. If you are agreeing to this license on behalf
 * of a company, you represent that you are authorized to bind the company to
 * such a license. If you do not meet this criterion or you do not agree to
 * any of the terms of this license, do NOT download, distribute, use or alter
 * this software or file in any way.
 *
 * Author(s): Burton Samograd <burton@wizard.ca>
 *            Josh Wilsdon <josh@wizard.ca>
 *
 * CVS Id: $Header: /web/programmers/cvs/magicmail-backend/magic-smtpd/magic-smtpd.c,v 1.51 2006/01/05 22:25:59 josh Exp $
 *
 * DO NOT MODIFY WITHOUT CONSULTING THE LICENSE
 */


/* XXX XXX XXX
 *
 * With debian make sure the "ca-certificates" package is installed
 *
 */


/* Standard C includes */
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/resource.h>
#include <sys/wait.h>
#include <time.h>
#include <fcntl.h>
#include <ctype.h>
#include <errno.h>
#include <sys/stat.h>
#include <signal.h>
#include <string.h>
#include <stdarg.h>

/* Includes that are required to properly be able to scan available ips that we accept user@[ip] for */

#include <sys/types.h>
#include <sys/param.h>
#include <sys/time.h>
#include <sys/ioctl.h>
#include <sys/socket.h>
#include <net/if.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <unistd.h>


/* Linux Magic library includes */
#include <liblm.h>

/* magic-smtpd includes */
#include "magic-smtpd.h"
#include "smtp.h"
// #include "cdb.h"
#include <freecdb/freecdb.h>
#include "valid_user_check.h"
#include "spam_check.h"
#include "smtp_auth.h"
#include "smtp-limits.h"
#include <common/stopwatch.h>
#include <common/spam_log.h>
#include <common/spam_rules.h>
#include <common/write_spam_log.h>
#ifdef USE_LM_UUID
#include <common/uuid.h>
#endif

#ifdef RELEASE_VERSION
#ifdef OPENSOURCE
    /* Opensource Release */
    #include <common/version.h.OPENSOURCE>
#else
    /* Commercial Release */
    #include <common/version.h.STABLE>
#endif
#else
    /* Development */
    #include <common/version.h>
#endif

/* NEW STYLE */
#include <common/spam_rules.h>
#include <common/controls.h>
#include <common/load_controls.h>
#include <common/spam_log.h>

#ifdef USE_TLS
/* STARTTLS */

#include <openssl/ssl.h>
#include <openssl/err.h>
#include "tls.h"

#ifndef OPENSOURCE
#include "msdstatus.h"
#endif

#ifdef USE_LM_DATABASE
#include <postgresql/libpq-fe.h>
#endif

int using_tls = 0;  /* Global flag for whether we are already using TLS */
char *encryption = NULL;
SSL *ssl;

#endif

/* missing prototypes */
int vasprintf(char **strp, const char *fmt, va_list ap);
int asprintf(char **strp, const char *fmt, ...);

int msd_quit_callback(int retval);
int get_domain(lm_string_t * pdomain, lm_string_t addr);
int get_user(lm_string_t * puser, lm_string_t addr);
int get_user_domain(lm_string_t * puser, lm_string_t * pdomain, lm_string_t addr);
int msd_timeout_read_line_lm(int fd, lm_string_t * line);
int checkRcptHost(lm_string_t domain);
int smtp_parse_address(lm_string_t string, lm_string_t * paddr, int *rfc_format);


spam_rule_t user_rules[] = SPAM_RULES;
spam_rule_t global_rules[] = SPAM_RULES;

/* for segfault handling */
char segfaultdir[] = "/var/cores/magic-smtpd/XXXXX";

/* Main configuration data, see config.h for contents */
// config_msd_t config;
config_t config;
config_type_t config_type_info[] = CONFIG_TYPE_DATA;
config_set_t config_set_info[] = CONFIG_SET_DATA;


/* Spam log structure */
extern spam_log_t spam_log;

/* Global readonly variables */
typedef struct tag_rcpt_data_t {
    lm_string_t addr;
    int spam_hit_flag;
} rcpt_data_t;

rcpt_data_t rcpt_list[100];
int rcpt_list_count;

int bounce_from = 0;

/* Global flag for whether things were formatted correctly */
int mail_from_rfc_formatted = 0;

/* Module static variables */
static lm_string_t smtp_from_addr;  /* from addrss for message */
static lm_string_t smtp_rcpt_list;  /* rcpt list of addresses for message */
static lm_string_t smtp_cmd;    /* current smtp command  */
static int smtp_message_size;   /* current message size */
static int smtp_message_size_overflow;  /* overflow bit for message size */
static int smtp_error_flag;     /* flag set if an error occures  */
static int spam_hit_flag = 0;       /* flag to say if on of the spam rules tested positive  */
static int penv[2];             /* envelope pipe for qmail queue */
static int pdata[2];            /* data pipe for qmail queue */
static int pout[2];             /* output pipe for qmail queue */
static int childpid;            /* pid of qmail-queue process */
static int num_hops;            /* number of hops this message has taken acccoring to Delivered and Recieved headers */
static int bad_mail_from_flag;  /* mail from: address was found in the badmailfrom list */
static int rcpt_bracket_ip_flag;    /* flag to say if we got a [d.d.d.d] address in rcpt */
static int num_invalid_rcpt;    /* number of rcpts which resulted in an invalid user  */
static int authenticated_flag;  /* flag to indicate if the user is authenticated */
static char *data_return_msg;   /* message to be printed at the end of the data callback, which may have error information */
static char *readbuf;           /* input buffer */
static int timeout_flag;        /* flag to indicate if reading input has timed out */
static int gotmail_flag;        /* flag to indicate if we have received a mail command */
static int rcpt_delay;          /* amount of seconds to delay when getting a rcpt */
static int rcpt_count;          /* number of valid user rcpts we have received */
static int whitelist = 0;       /* 0 if not on whitelist 1 if on whitelist */
static int mailfrom_local_domain = 0;   /* track whether MAIL FROM is in rcpthosts */
int fakehelohost;
int rcptnodelay = 0;

/* Constant strings which are all allocated at the start */

static lm_string_t T = NULL;    /* constant T for rcpt address list */
static lm_string_t F = NULL;    /* constant F for from address */
static lm_string_t atsign = NULL;   /* constant @ */
lm_string_t helohost = NULL;    

/* missing prototypes */
int vasprintf(char **strp, const char *fmt, va_list ap);
int asprintf(char **strp, const char *fmt, ...);

int msd_quit_callback(int retval);
int get_domain(lm_string_t * pdomain, lm_string_t addr);
int get_user(lm_string_t * puser, lm_string_t addr);
int get_user_domain(lm_string_t * puser, lm_string_t * pdomain, lm_string_t addr);
int msd_timeout_read_line_lm(int fd, lm_string_t * line);
int lmprintf(const char *format, ...);

int lmprintf(const char *format, ...)
{
    va_list ap;
    int ret;
#ifdef USE_TLS
    char *tmpstr;
    int timeout, wbytes;
#endif

    ret = -1;
    va_start (ap, format);

#ifdef USE_TLS
    if (using_tls == 1) {
        ret = vasprintf(&tmpstr, format, ap);
        if (ret > 0) {
            wbytes = ssl_timeoutio(SSL_write, tmpstr, strlen(tmpstr), &timeout);
            if (wbytes == 0) {
                /* remote disconnected, can't send error */
                syslog(LOG_INFO, "lmprintf() remote disconnected");
                msd_quit_callback(1);
                /* NOTREACHED */

            } else if (wbytes < 0) {
                /* failed with an error */
                if (timeout == 1) {
                     /* can't write to client because that's what timed out */
                    syslog(LOG_ERR, "lmprintf() timed out");
                    msd_quit_callback(1);
                    /* NOTREACHED */

                } else {
                    /* XXX
                     * The cases where we can reach this need to be listed 
                     * Can't write to client, because that was the problem
                     */
                     syslog(LOG_INFO, "lmprintf() internal error");
                     msd_quit_callback(1);
                     /* NOTREACHED */

                }
            } 

            /* succeeded */

            /* XXX we are assuming that all bytes were written */
        }
    } else {
#else 
    if (1) {
#endif
        ret = vfprintf (stdout, format, ap);
        fflush(stdout);
    }
    va_end (ap);

    return (ret);
}


int msd_timeout_read_line_lm(int fd, lm_string_t * line)
{
    int ret, len;
    char *tmp = NULL;

    ret = msd_timeout_read_line(fd, &tmp, &len);
    if (ret == 0) {
        LM_STRING_NEW_LEN(*line, tmp, len);
    }

    if (tmp != NULL) {
        free(tmp);
    }

    return (ret);
}


/* Check to see if 'checkip' matches any ips that are on our network. */

int isIPinList(char *checkip) {

    struct ifconf ifc;
    char buff[8192];
    int fd, i, n;
    struct ifreq *ifr=NULL;
    int total = 0;
    struct in_addr ipaddr;
    //struct in_addr nmask;
    //char *iname;

    int MAX_CHECKED = 100; // out of all available ips, which should be few, just check 100.
    
    if ((fd = socket(AF_INET, SOCK_DGRAM, 0)) == -1) {
        return -1;
    }

    ifc.ifc_len = sizeof(buff);
    ifc.ifc_buf = buff;

    if (ioctl(fd, SIOCGIFCONF, &ifc) != 0) {
        close(fd);
        return -1;
    }

    ifr = ifc.ifc_req;
    n = ifc.ifc_len / sizeof(struct ifreq);

    /* Loop through interfaces, looking for given IP address */
    for (i=n-1;i>=0 && total < MAX_CHECKED ;i--) {
        if (ioctl(fd, SIOCGIFADDR, &ifr[i]) != 0) {
            continue;
        }

        //iname = ifr[i].ifr_name;
        ipaddr = (*(struct sockaddr_in *)&ifr[i].ifr_addr).sin_addr;

        if (ioctl(fd, SIOCGIFFLAGS, &ifr[i]) != 0) {
            continue;
        }

        if (!(ifr[i].ifr_flags & IFF_UP)) {
            continue;
        }

        if (ioctl(fd, SIOCGIFNETMASK, &ifr[i]) != 0) {
            continue;
        }
    
        if(strcmp(checkip, inet_ntoa(ipaddr)) == 0) {
            // woo hoo, a match.
            close(fd);
            return 1;
        }        
        //nmask = ((struct sockaddr_in *)&ifr[i].ifr_addr)->sin_addr;
        //printf("%s\n", inet_ntoa(ipaddr.s_addr));
        //total++;
    }
    close(fd);
    return 0;
}



/* Handle resettting of all internal variables */
int msd_rset_callback(void)
{
    int retval;

    /* Free the old data strings */
    LM_STRING_FREE(smtp_rcpt_list);
    LM_STRING_FREE(smtp_from_addr);

    /* Reset the strings to empty */
    retval = 0;
    LM_STRING_NEW(smtp_rcpt_list, "");
    retval = (smtp_rcpt_list == NULL);
    LM_STRING_NEW(smtp_from_addr, "");
    retval |= (smtp_from_addr == NULL);

    if (retval != 0) {
        msd_error_out_of_memory();
    }

    /* Reset the message size and overflow flag to zero */
    smtp_message_size = 0;
    smtp_message_size_overflow = 0;

    /* Reset the error flag */
    smtp_error_flag = 0;

    /* Reset data callback return message string */
    data_return_msg = 0;

    /* Reset the bad mail from flag */
    bad_mail_from_flag = 0;

    /* Reset the rcpt count callbacks */
    rcpt_count = 0;

    /* Reset the authenticated flag */
    authenticated_flag = 0;

    /* Reset the gotmail flag */
    gotmail_flag = 0;

    /* Reset the spam hit flag */
    spam_hit_flag = 0;

    /* Reset whitelist flag */
    whitelist = 0;

    /* Reset the bounce_from flag */
    bounce_from = 0;

    return 0;
}

/* Perform cleanup operations and exit */
int msd_quit_callback(int retval)
{
    if (retval == 0) {
        lmprintf("221 %s\r\n", config.me);
    }

    /* Do cleanup operations */
    /* config_msd_cleanup(); */
    /* config_qmail_cleanup(); */
    smtp_cleanup();

    if (smtp_cmd != NULL) {
        LM_STRING_FREE(smtp_cmd);
    }

    /* Free any data that has been read */
    LM_STRING_FREE(smtp_rcpt_list);
    LM_STRING_FREE(smtp_from_addr);
    LM_STRING_FREE(F);
    LM_STRING_FREE(T);
    LM_STRING_FREE(atsign);
    free(readbuf);

    /* DISABLED by JW if (config.log_level >= LOG_DEBUG) {
        LM_MEMSTAT(stderr);
        LM_MEMDUMP(stderr);
     */

    syslog(LOG_INFO, "Exiting");
    closelog();

    fflush(stdout);
    fflush(stderr);

#ifdef USE_TLS
    if (using_tls == 1)  {
        SSL_shutdown(ssl);
        SSL_free(ssl);
    }
#endif

    exit(retval);
    /* prevents compiler warnings */
    return retval;
}

void msd_error_unavailable(void)
{
    lmprintf("421 Service not available, closing transmission channel\r\n");
    /* Just exit here since this can be called before things are initialized and that
       causes problems.  No real need to call cleanup functions anyways since all memory 
       will be freed on exit */

    syslog(LOG_ALERT, "returning service unavailable");
    syslog(LOG_INFO, "Exiting");
    closelog();
    exit(1);
    /*  msd_quit_callback(1); */
}

void msd_error_out_of_memory(void)
{
    /* Print no mem error message */
    lmprintf("421 out of memory (#4.3.0)\r\n");

    /* Die with exit code 1 */
    syslog(LOG_EMERG, "out of memory");
    msd_quit_callback(1);
}

/* Exec the qmail-queue and setup write pipes to send the envelope and 
   body data as they are provided by the queue callback */
int msd_queue_init(void)
{
    char *args[] = { NULL, NULL };
    char *buffer = NULL;
    char *local;
#ifdef USE_LM_UUID
    char *uuid;
#endif
    int buffersize;
    int numchars;
#ifndef OPENSOURCE
    char *msdstatusstr;

    unsetenv("MSDSTATUS"); /* make sure something doesn't leak in */
    msdstatusstr = getMsdStatusLine();
    if (msdstatusstr != NULL) {
        setenv("MSDSTATUS", msdstatusstr, 1);
        syslog(LOG_DEBUG, "MSDSTATUS: [%s]", getenv("MSDSTATUS"));
    }
    cleanupMsdStatus();
    if (msdstatusstr != NULL) {
        free(msdstatusstr);
    }
#endif

#ifdef USE_LM_UUID
    unsetenv("MSDUUID");
    uuid = uuid_v1();
    if (uuid != NULL) {
        syslog(LOG_INFO, "Created UUID %s for message", uuid);
        setenv("MSDUUID", uuid, 1);
        free(uuid);
    }
#endif

    args[0] = config.qmail_queue;

    /* Allocate the read buffer */
    buffersize = config.max_line_length;
    if ((buffer = malloc(buffersize)) == NULL) {
        LM_ASSERT(buffer == NULL);
        msd_error_out_of_memory();
    }

    /* Reset the number of hops for new message */
    num_hops = 0;

    if ((pipe(penv) == -1) || (pipe(pdata) == -1) || (pipe(pout) == -1)) {
        syslog(LOG_DEBUG, "pipe error: %s", strerror(errno));
    }

    switch (childpid = fork()) {
        case 0:
            /* Child */

            /* close descriptor 0 and re-open it as the read end of pdata */
            close(0);
            dup(pdata[0]);

            /* close the pdata pipe (we've still got descriptor 0 pointing to it) */
            close(pdata[0]);
            close(pdata[1]);

            /* close descriptor 1 and re-open it as the read end of penv */
            close(1);
            dup(penv[0]);

            /* close the penv pipe (we've still got descriptor 1 pointing to it) */
            close(penv[0]);
            close(penv[1]);

           /* close stderr and re-open as write end of pout */
           close(2);
           dup(pout[1]);

           /* close pout (stderr points to it) */
           close(pout[0]);
           close(pout[1]);

            /* run qmail-queue (which reads from descriptors 0 and 1) */
            /* TODO Is this safe? Might need to reconsider executing a string that is 
             * pulled from the environment 
             */
            if (execvp(config.qmail_queue, args)) {
                syslog(LOG_DEBUG, "execvp: %s", strerror(errno));
                return -1;
            }
            /* should never get here */
            syslog(LOG_DEBUG, "execvp unreachable: %s", strerror(errno));
            _exit(1);
            break;

        case -1:
            /* fork() error */
            syslog(LOG_ERR, "qmail-queue fork error");
            break;
    }

    close(pout[1]);

    /* Write the Recieved: line from us for the message */
    numchars = snprintf(buffer, buffersize, "Received: from %s ", config.tcpremotehost);
    if ((smtp_error_flag == 0) && (numchars > 0)
        && (write(pdata[1], buffer, numchars) != numchars)) {
        smtp_error_flag |= 1;
    }

    /* If we were given a helo domain different than tcp server, print it */
    if (fakehelohost != 0) {
        numchars = snprintf(buffer, buffersize, "(HELO %s) ",
                     LM_STRING_BUFFER(helohost));
        if ((smtp_error_flag == 0) && (numchars > 0)
            && (write(pdata[1], buffer, numchars) != numchars)) {
            smtp_error_flag |= 1;
        }
    }

    /* Write an ( for the remote host information to be contained in */
    numchars = snprintf(buffer, buffersize, "(");
    if ((smtp_error_flag == 0) && (numchars > 0)
        && (write(pdata[1], buffer, numchars) != numchars)) {
        smtp_error_flag |= 1;
    }

    /* Write the remote info if given by tcpserver */
    if ((config.tcpremoteinfo != NULL) && (config.tcpremoteinfo[0] != '\0')) {
        numchars = snprintf(buffer, buffersize, "%s@", config.tcpremoteinfo);
        if ((smtp_error_flag == 0) && numchars > 0
            && write(pdata[1], buffer, numchars) != numchars) {
            smtp_error_flag |= 1;
        }
    }

    /* this is what qmail does */
    local = getenv("TCPLOCALHOST");
    if (local == NULL) {
        local = config.tcplocalip;
    } 
    if (local == NULL) {
        local = "unknown";
    }

    /* Write the remote ip */
#ifdef USE_TLS
    numchars = snprintf(buffer, buffersize, "%s)\n\tby %s with %s; "
                 , config.tcpremoteip
                 , local
                 , encryption ? encryption : "SMTP");
#else
    numchars = snprintf(buffer, buffersize, "%s)\n\tby %s with SMTP; ", config.tcpremoteip, local);
#endif

    if ((smtp_error_flag == 0) && numchars > 0
        && write(pdata[1], buffer, numchars) != numchars) {
        smtp_error_flag |= 1;
    }

    /* Print out the time in the received line */
    {
        time_t t;
        struct tm *times;

        time(&t);

        /* 
         * Print out the time string according to the RFC and man strftime
         */
        times = localtime(&t);
        numchars = strftime(buffer, buffersize, "%a, %d %b %Y %H:%M:%S %z\n", times);
        if ((smtp_error_flag == 0) && numchars > 0
            && write(pdata[1], buffer, numchars) != numchars) {
            smtp_error_flag |= 1;
        }
    }

    /* Free the buffer */
    free(buffer);
    return 0;
}


/* Write the envelope data to qmail-queue  */
int msd_queue(void)
{
    int index;
    int retval;

    /* Need to close the message body pipe so that we can write to the header
       data pipe */
    close(pdata[1]);
 
    /* Write the envelope to file descriptor 1 */
 
    /* Write the from address and it's trailing \0 to the message envelope pipe */
    LM_STRING_ASSERT(smtp_from_addr);
    smtp_error_flag |= (LM_STRING_CHECK(smtp_from_addr) != 0);
    if (smtp_error_flag == 0) {
        retval = write(penv[1], "F", 1);
        smtp_error_flag |= (retval != 1);
    }

    /* Write the from addr to the envelope pipe */
    if (smtp_error_flag == 0) {
        retval = write(penv[1], LM_STRING_BUFFER(smtp_from_addr),
                  LM_STRING_LEN(smtp_from_addr));
        smtp_error_flag |= (retval != LM_STRING_LEN(smtp_from_addr));
    }

    /* Write the terminator character for the from address */
    if (smtp_error_flag == 0) {
        retval = write(penv[1], "\0", 1);
        smtp_error_flag |= (retval != 1);
    }

    /* QMAIL */
    /* If there was an error don't write the rcpt list */
    if (smtp_error_flag) {
        syslog(LOG_ERR, "error writing envelope from section!");
        return 0;
    }

    /* Find the location of the first T in the rcpt list */
    if (LM_STRING_CHR(smtp_rcpt_list, 'T', index)) {
        /* If there isn't one, there was a serious problem */
        syslog(LOG_ERR, "empty rcpt list!!!");
        return -1;
    }

    /* index should always be 0, since all rcpt addresses start with a T */
    LM_ASSERT(index == 0);
    while (index == 0) {
        /* Shift off the T */
        LM_ASSERT(LM_STRING_BUFFER(smtp_rcpt_list)[0] == 'T');
        LM_STRING_START_SHIFT(smtp_rcpt_list, 1);

        /* Write a T to the pipe */
        if (smtp_error_flag == 0) {
            retval = write(penv[1], "T", 1);
            smtp_error_flag |= (retval != 1);
        }

        /* Find the next T */
        if (LM_STRING_CHR(smtp_rcpt_list, 'T', index)) {
            /* If there wasn't another T, we are at the last address in the list,
               so we write that and then put the two null terminators to signify the
               end of the envelope */
            if (smtp_error_flag == 0) {
                retval =
                    write(penv[1], LM_STRING_BUFFER(smtp_rcpt_list),
                          LM_STRING_LEN(smtp_rcpt_list));
                smtp_error_flag |= (retval != LM_STRING_LEN(smtp_rcpt_list));
            }

            /* this is actually 3 bytes long */
            if (smtp_error_flag == 0) {
                retval = write(penv[1], "\0\0", 2);
                smtp_error_flag |= (retval != 2);
            }
            break;
        } else {
            /* We found another T, so we write the address up to where the T was found 
               and then write a single null terminator and then shift that address off
               the start of the string  */
            LM_ASSERT(index <= LM_STRING_LEN(smtp_rcpt_list));
            if (smtp_error_flag == 0) {
                retval =
                    write(penv[1], LM_STRING_BUFFER(smtp_rcpt_list), index);
                smtp_error_flag |= (retval != index);
            }

            /* this is actually 2 bytes long */
            if (smtp_error_flag == 0) {
                retval = write(penv[1], "\0", 1);
                smtp_error_flag |= (retval != 1);
            }

            LM_STRING_START_SHIFT(smtp_rcpt_list, index);
            index = 0;
        }
    }

    return 0;
}

/* Close the pipes to the qmail-queue process that are still open, and wait for the 
 * child to exit and get it's exit code */
int msd_queue_cleanup(void)
{
    int wpret, n, i;
    char childret;
    int retval;
    char output[1024];

    /* Close the pipe */
    close(penv[1]);

    while ((n = read(pout[0], output, sizeof(output) - 1)) > 0) {
       for (i = 0; i < n; i++) {
           if (!(isprint(output[i]) || output[i] == ' ')) {
               if (i < (n - 1)) {
                   output[i] = '/';
               } else {
                   /* in this case, the second last character is bad (probably
                    * a newline), so we ignore it completely. */
                   output[i] = '\0';
               }
           }
       }

       output[n] = '\0';
       syslog(LOG_INFO, "Virus scan: %s", output);
    }

    if (n == -1) {
       syslog(LOG_ERR, "error reading from qq: %s\n", strerror(errno));
    }

    /* Wait for the child to stop */
    do {
        wpret = waitpid(childpid, &retval, 0);
    } while (wpret == -1 && errno == EINTR);

    /* Check if there was some sort of wierd problem with waitpid */
    if (wpret != childpid) {
        /* TODO not tested */
        data_return_msg = "451 qq waitpid surprise (#4.3.0)\r\n";
        return 0;
    }

    /* Check to see if qmail-queue crashed */
    if (retval & 127) {
        /* TODO not tested */
        data_return_msg = "451 qq crashed (#4.3.0)\r\n";
    }

    /* Check the exti status of qmail queue and print the appropriate return message */
    /* I have bad feelings about WEXITSTATUS returning 0-255. 
     * This will make sure the value is sign extended to an int
     */
    childret = WEXITSTATUS(retval);
    retval = childret;
    if (retval > 0) {
        switch (retval) {
            case 115:
            case 11:
                data_return_msg = "554 envelope address too long for qq (#5.1.3)";
                break;
            case 31:
                data_return_msg = "554 mail server permanently rejected message (#5.3.0)";
                break;
            case 32:
                data_return_msg = "554 rejected by virus scanner (#5.3.0)";
                syslog(LOG_WARNING, "DATA rejected by virus scanner");
                break;
            case 51:
                data_return_msg = "451 qq out of memory (#4.3.0)";
                break;
            case 52:
                data_return_msg = "451 qq timeout (#4.3.0)";
                break;
            case 53:
                data_return_msg = "451 qq write error or disk full (#4.3.0)";
                break;
            case 0:
                if (!smtp_error_flag) {
                    break;
                }
                /* fall through if there was an error */
            case 54:
                data_return_msg = "451 qq read error (#4.3.0)";
                break;
            case 55:
                data_return_msg = "451 qq unable to read configuration (#4.3.0)";
                break;
            case 56:
                data_return_msg = "451 qq trouble making network connection (#4.3.0)";
                break;
            case 61:
                data_return_msg = "451 qq trouble in home directory (#4.3.0)";
                break;
            case 63:
            case 64:
            case 65:
            case 66:
            case 62:
                data_return_msg = "451 qq trouble creating files in queue (#4.3.0)";
                break;
            case 71:
                data_return_msg = "451 mail server temporarily rejected message (#4.3.0)";
                break;
            case 72:
                data_return_msg = "451 connection to mail server timed out (#4.4.1)";
                break;
            case 73:
                data_return_msg = "451 connection to mail server rejected (#4.4.1)";
                break;
            case 74:
                data_return_msg = "451 communication with mail server failed (#4.4.2)";
                break;
            case 91:           /* fall through */
            case 81:
                data_return_msg = "451 qq internal bug (#4.3.0)";
                break;
            case 120:
                data_return_msg = "451 unable to exec qq (#4.3.0)";
                break;
            default:
                if ((retval >= 11) && (retval <= 40)) {
                    data_return_msg = "554 qq permanent problem (#5.3.0)";
                    break;
                }
                data_return_msg = "451 qq temporary problem (#4.3.0)";
                break;
        }
    }

    return retval > 0;
}

/* Check if this message has been bounced or relayed and increment the hop
   count if it has */
void msd_header_check_hop(char *header_line)
{
    char received[] = "Received";
    char delivered[] = "Delivered";

    /*
     * Look for any "Received:" or "Delivered:" headers.  For each of these
     * we'll increment the hop count so that we can detect loops.
     *
     */
    if ((strncasecmp(received, header_line, strlen(received)) == 0)
        || (strncasecmp(delivered, header_line, strlen(delivered)) == 0)) {
        syslog(LOG_INFO, "incrementing hop count");
        num_hops++;
    }

    /* If the number of hops was too great print warning */
    if ((num_hops > config.max_hops) && (config.max_hops > 0)) {
        syslog(LOG_WARNING, "max hop count exceeded!");

        /* Flag an error for to be caught later */
        smtp_error_flag |= 1;

        lmprintf("554 too many hops, this message is looping (#5.4.6)\r\n");
        msd_quit_callback(1);
    }
}

int msd_update_msg_size(int size)
{
    unsigned int tmpsize;

    /* Add the length of the line to the total message size */
    /* Check for overflow of smtp_message_size */
    tmpsize = smtp_message_size;
    tmpsize += size;

    /* If the value of tmpsize is less than the old message size,
     * there was a size overflow so we'll flag an error
     */
    if ((unsigned int) tmpsize > (unsigned int) smtp_message_size) {
        smtp_message_size = tmpsize;
        if (config.databytes != 0) {
            if (smtp_message_size > config.databytes) {
                smtp_error_flag |= 1;
                return -1;
            }
        }
    } else {
        /* Overflow in size variable error */
        smtp_message_size_overflow = 1;
        smtp_error_flag |= 1;
    }

    return 0;

}

/* This function is called for each line contained in the header
   Simply writes it to the qmail-queue message pipe for now */
int msd_header_line_callback(lm_string_t line)
{
    int retval = 0;

    /* Update the total number of bytes that have been received */
    msd_update_msg_size(LM_STRING_LEN(line));

    /* Perform hop checking like qmail does */
    msd_header_check_hop(LM_STRING_BUFFER(line));

    /* If there hasn't been an error yet, write the line to qmail-queue */
    if (smtp_error_flag == 0) {
        retval = write(pdata[1], LM_STRING_BUFFER(line), LM_STRING_LEN(line));
    } else {
        /* There was an error */

    }

    /* Flag an error if line wasn't written fully */
    smtp_error_flag |= (retval != LM_STRING_LEN(line));

    return 0;
}

/* This function is called for each line contained in the header
   Simply writes it to the qmail-queue message pipe for now */
int msd_body_line_callback(lm_string_t line)
{
    int retval = 0;

    /* Update the total number of bytes that have been received */
    msd_update_msg_size(LM_STRING_LEN(line));

    /* Write the line to the data pipe */
    if (!smtp_error_flag) {
        retval = write(pdata[1], LM_STRING_BUFFER(line), LM_STRING_LEN(line));
    }

    /* Flag an error if line wasn't written fully */
    smtp_error_flag |= (retval != LM_STRING_LEN(line));

    return 0;
}

/* Perform authorization with the given username and password, returning
   0 on sucess or
   * -1 on failure and setting the authorized_flag to true */
int msd_auth_callback(int authtype, lm_string_t username64)
{
    int retval = 0;

    /* If auth is not enabled, print error */
    if (config.auth_enable == 0) {
        lmprintf("503 auth not available (#5.3.3)\r\n");
        return -1;
    }

    /* If we've already gotten a mail command, can't do an auth */
    if (gotmail_flag != 0) {
        lmprintf("503 no auth during mail transaction (#5.5.0)\r\n");
        return -1;
    }

    /* If already authorized, print error */
    if (authenticated_flag != 0) {
        lmprintf("503 you're already authenticated (#5.5.0)\r\n");
        return -1;
    }

    /* Do authorization */
    retval = msd_do_auth(authtype, username64);

    /* Check the return code and set retval accordingly */
    if (retval == 0) {
        /* Print authorization denedied message and set retval to -1 */
        authenticated_flag = 1;
        setenv("RELAYCLIENT", "", 0);
        lmprintf("235 ok, go ahead (#2.0.0)\r\n");
    } else if (retval > 0) {
        /* Authorization denied, print error */
        lmprintf("535 authorization failed (#5.7.0)\r\n");
        syslog(LOG_NOTICE, "Sleeping due to auth failure");
        sleep(5);
    } else {
        switch (retval) {
            case LM_AUTH_UNSUPPORTED:
                lmprintf("504 Unsupported authorization type.\r\n");
                break;
            case LM_AUTH_ERROR_DB:
                lmprintf("454 Temporary authentication failure.\r\n");
                break;
            case LM_AUTH_CANCEL:
                lmprintf("501 Authorization canceled\r\n");
                break;
            case LM_AUTH_BAD_DATA:
                lmprintf("501 Username or password cannot be decoded.  Authorization canceled.\r\n");
                break;
            case LM_AUTH_ERROR:
            default:
                lmprintf("454 problems performing authorization (#4.3.0)\r\n");
                break;
        }
        syslog(LOG_NOTICE, "Sleeping after auth problem/failure");
        sleep(5);
    }

    /* Return non-zero to prevent further processing by caller */
    return 1;
}

/* Function called after all queuing functions have completed */
int msd_data_callback(void)
{
    /* Check if the message size is greater than maximum size and print 
     * error return code if
     * Also check if there was an overflow.
     */
    if ((smtp_message_size > config.databytes)
        || smtp_message_size_overflow) {
        syslog(LOG_WARNING, "message size greater than databytes!");
        lmprintf("552 sorry, that message size exceeds my databytes limit (#5.3.4)\r\n");
        return 0;
    }

    /* Print the current time and the pid of the child process for return
       code message string */
    if (data_return_msg) {
        syslog(LOG_DEBUG, "Data returning: %s", data_return_msg);
        lmprintf("%s\r\n", data_return_msg);
    } else {
        syslog(LOG_INFO, "Returning 250 ok [qp %d] for data", childpid);
        lmprintf("250 ok %d qp %d\r\n", (int) time(NULL), childpid);
    }

    return 0;
}


/* Check to see if the given address is in the bad mail from list */
int msd_bad_mail_from(lm_string_t addr)
{
    char *string;
    list_t *ptr;

    string = LM_STRING_BUFFER(addr);
    ptr = config.badmailfrom;
    while (ptr != NULL) {
        if (strncasecmp(ptr->data, string, strlen(ptr->data) + 1) == 0) {
            /* match */
            return (1);
        }

        ptr = ptr->next;
    }

    /* not found */
    return (0);
}


/* Function called after all queuing functions have completed */
int msd_mail_callback(lm_string_t addr)
{
    lm_string_t domain = NULL;
    int retval = 0;

    /* Perform a reset when this is called */
    msd_rset_callback();

    /* Set the gotmail flag for the authentication part */
    gotmail_flag = 1;

    /* TODO parse out the address */

    /* Check the address to see if it's in the badmailfrom list */
    if (msd_bad_mail_from(addr) != 0) {
        bad_mail_from_flag = 1;
        return 0;
    }

    if ((config.check_valid_users > 0) && (config.check_valid_from > 0)) {

        /* Get the domain part of the address */
        if (get_domain(&domain, addr)) {
            msd_error_out_of_memory();
        }
    
        /* If the address doesn't have a domain no need to check rcpt hosts */
        if (LM_STRING_LEN(domain) > 0) {
    
            /* if domain is in rcpthosts check MAIL FROM: for valid user */
            if (checkRcptHost(domain) == 0) {
                mailfrom_local_domain = 1;
                retval = msd_check_rcpt_user(&addr);
                if (retval > 0) {
                    /* invalid */
                    syslog(LOG_INFO, "rejecting non-existent local sender");
                    lmprintf("553 sorry, you are not allowed to send through this server (#5.7.1)\r\n");
                    LM_STRING_FREE(domain);
                    return (1);
                }
            }
        }
        
        LM_STRING_FREE(domain);
    }

    /* Save the parsed address */
    LM_STRING_FREE(smtp_from_addr);
    if (LM_STRING_DUP(smtp_from_addr, addr) != 0) {
        msd_error_out_of_memory();
    }

    if (authenticated_flag > 0) {
        syslog(LOG_INFO, "Allowed to relay: yes (SMTP AUTH)");
    } else if (config.tcprelayclient != NULL) {
        syslog(LOG_INFO, "Allowed to relay: yes (RELAYCLIENT)");
    } else {
        syslog(LOG_INFO, "Allowed to relay: no");
    }

    return (0);
}


/* 
 * SUMMARY
 *
 *  Checks whether a domain is in the rcpthosts or morercpthosts.cdb files.
 *
 * RETURN VALUE
 *
 *  0   found in rcpthosts or morercpthosts.cdb
 *  1   not found in rcpthosts or morercpthosts.cdb
 * -1   an error occured during checks 
 *  
 */
int checkRcptHost(lm_string_t domain)
{   
    int retval = 0;
    char *string;
    list_t *ptr;
    int cdb_fd, pattern_len, string_len, i;
    char morercpthosts_path[1024];

    string = LM_STRING_BUFFER(domain);
    ptr = config.rcpthosts;
    while (ptr != NULL) {
        if (ptr->data != NULL) {
            if (strncasecmp(ptr->data, string, strlen(ptr->data) + 1) == 0) {
                /* match */
                syslog(LOG_DEBUG, "rcpthost [%s] matched [%s]", string, ptr->data);
                return (0);
            }
    
            /* check for wildcard domain (staring with a '.') */
            if (ptr->data[0] == '.') {
                pattern_len = strlen(ptr->data);
                string_len = strlen(string);
                if (string_len > pattern_len) {
                    for (i = 1; i < (pattern_len + 1); i++) {
                        if (string[string_len - i] != ptr->data[pattern_len - i]) {
                            break; 
                        }
                    }
                    if (i > pattern_len) {
                        /* match */
                        syslog(LOG_DEBUG, "rcpthost [%s] matched wildcard [%s]", string, ptr->data);
                        return (0);
                    }
                }
            }
        }
    
        ptr = ptr->next;
    }

    /* not found in rcpthosts */
    retval = 1;

    strcpy(morercpthosts_path, "/var/qmail/control/");
    strcat(morercpthosts_path, "morercpthosts.cdb");
    syslog(LOG_DEBUG, "checking morercpthosts.cdb for %s\n", LM_STRING_BUFFER(domain));
    if((cdb_fd = open(morercpthosts_path, O_RDONLY)) == -1) {
        syslog(LOG_DEBUG, "could not open morercpthosts.cdb file");
    } else {
        retval = config_check_morercpthosts(cdb_fd, LM_STRING_BUFFER(domain));      /* returns 1 if found */
        syslog(LOG_DEBUG, "morercpthosts.cdb: %d returned for %s\n", retval, LM_STRING_BUFFER(domain));
        switch (retval) {
            case 1:
                retval = 0; /* found */
                break;
            case 0:     /* not found in morercpthosts.cdb */
                /* FALLTHROUGH */
            case -1:    /* error reading morercpthosts.cdb, treat as not found for now */
                /* FALLTHROUGH */
            default:
                retval = 1; /* not found */
                break;
        }
    }

    return (retval);    /* 0 = found, 1 = not found, -1 = error */
}                   


/* Check the given hostname to see if it is contained in the rcpthosts 
 * file and we should accept the mail 
 */
int msd_rcpt_host(lm_string_t * paddr)
{
    lm_string_t domain = NULL, defaultdomain = NULL;
    int retval = 0;
    lm_string_t addr;

    LM_STRING_ASSERT(*paddr);
    addr = *paddr;

    /* Reset the bracket IP flag for each RCPT */
    rcpt_bracket_ip_flag = 0;

    /* Get the domain part of the address */
    if (get_domain(&domain, addr)) {
        msd_error_out_of_memory();
    }

    /* If the address doesn't have a domain
       no need to check rcpt hosts */
    if (LM_STRING_LEN(domain) == 0) {
        LM_STRING_FREE(domain);
        return 0;
    }

    /* Check for local ip address after @ sign that matches
     * machine ip, which would also be local delivery */
    /* THIS IS BROKEN AND ASSUMES ALL @[...] ADDRESSES ARE
       TO THE LOCAL MACHINE!!!! */
    if (LM_STRING_BUFFER(domain)[0] == '[') {
        syslog(LOG_DEBUG, "Recieved mail with [ip] domain part: %s", LM_STRING_BUFFER(addr));

        /* HACK Set the bracket IP flag so that we don't 
           do any valid user or spam checking later */
        rcpt_bracket_ip_flag = 1;

        if (LM_STRING_BUFFER(domain)[LM_STRING_LEN(domain) - 1] != ']') {
            /* no close bracket */
            LM_STRING_FREE(domain);
            return -1;
        }

        LM_STRING_START_SHIFT(domain, 1);
        LM_STRING_END_SHIFT(domain, 1);

        /* NOTE: We only use the IP given in localiphosts instead
           of building the interface list list qmail-smtpd */

        /* Check to see if the IP matches our localiphost */
        //if (!LM_STRING_CMP(domain, config_qmail.localiphost)) {
        if(isIPinList(LM_STRING_BUFFER(domain))) {                         // i don't want lmstring, i want char *
            lm_string_t user = NULL;

            /* Build the new email address with the value of 
               localiphost as the domain */
            get_user(&user, addr);
            LM_STRING_FREE(addr);
            LM_STRING_NEW(defaultdomain, config.defaultdomain);
            if (LM_STRING_CAT(user, atsign)
                || LM_STRING_CAT(user, defaultdomain)
                || LM_STRING_DUP(addr, user)) {
                msd_error_out_of_memory();
            }

            syslog(LOG_DEBUG, "IP matched localiphost.  Delivering to: %s",
                    LM_STRING_BUFFER(addr));

            *paddr = addr;
            LM_STRING_FREE(user);
            
            retval = 0;
        } 
        else {
            /* The IP didn't match. */
            syslog(LOG_DEBUG, "IP didn't match localiphost");

            /* Check if relayclient is set, and if it is, just pass
               the mail through to qmail, else don't accept */
            if (config.tcprelayclient != NULL) {
                /* Relayclient set, return ok */
                syslog(LOG_DEBUG, "relayclient set, accepting mail for host");
                retval = 0;
            } else {
                /* Relayclient not set, don't deliver */
                syslog(LOG_DEBUG,
                        "relayclient not set, rejecting delivery for host: %s",
                        LM_STRING_BUFFER(domain));
                retval = -1;
            }
        }
        if (domain) {
            LM_STRING_FREE(domain);
        }
        return retval;
    }

    /* check rcpthosts and morercpthosts.cdb */
    retval = checkRcptHost(domain);

    LM_STRING_FREE(domain);

    return retval;                                              /* 0 if found, 1 if not found */
}

/*************************************************************/
/* Bart Dec 31, 2003 (new year's eve). Gettin drunk tonight. */
/* This function makes use of the freecdb code               */
/*************************************************************/

/* returns:
 *
 *  1 = found
 *  0 = not found
 *
 */
int config_check_morercpthosts(int cdb_file, char *domain) {

    uint32_t len;

    if(domain == NULL) {
	    syslog(LOG_DEBUG, "config_check_morercpthosts(): domain is NULL");
	    return (0);
    }

    switch(cdb_seek(cdb_file, domain, strlen(domain), &len)) {
        case -1:
            syslog(LOG_DEBUG, "cdbget: fatal: %s", strerror(errno));
	    /* FALLTHROUGH */
        case 0:
            return 0;               /* NOT FOUND IN CDB */
    }

    return 1; /* found */
}


enum rcpt_type findRcptType(lm_string_t addr)
{
    int retval;

    if (msd_rcpt_host(&addr) != 0) {
        /* if it's not in rcpthosts, it's definitely remote */
        return (RCPT_REMOTE);
    } 

    if (config.check_valid_users == 0) {
        syslog(LOG_INFO, "Valid user checking is not enabled");
        return (RCPT_LOCAL_UNCHECKED);
    }   

    if (rcpt_bracket_ip_flag != 0) {
        syslog(LOG_INFO, "Address is in the form user@[X.X.X.X], not doing valid user check");
        return (RCPT_LOCAL_UNCHECKED);
    }

    retval = msd_check_rcpt_user(&addr);
    if (retval > 0) {
        return (RCPT_LOCAL_NOEXIST);
    } else if (retval < 0) {
        syslog(LOG_ERR, "Error checking for valid user");
        return (RCPT_LOCAL_ERROR);
    }

    /* RCPT is local and exists if we are here */

    return (RCPT_LOCAL);
}
    


/* XXX Oct 22, 2003: Josh
 *
 *  This function does way too much.  Needs to be split up.
 *
 *     Dec 8, 2005: Josh
 *
 *  Cleaned up some, but still way way way too long.
 *
 */
int msd_rcpt_callback(lm_string_t args)
{
    int retval = 0, local;
    int spam_check_retval = 0;
    lm_string_t addr = NULL;
    int spam_rules_loaded_flag = 0;
    lm_string_t user, domain;
    stopwatch_t sw;
    enum rcpt_type type;
    lm_string_t rcptaddr = NULL;
    lm_string_t relayclient = NULL;
    char *typestr;

    /* this is not currently used outside this function */
    int rcpt_to_rfc_formatted;

    /* Get the address from the arguemnts, printing error message if syntax is incorrect */
    if (smtp_parse_address(args, &rcptaddr, &rcpt_to_rfc_formatted)) {
        syslog(LOG_NOTICE, "RCPT command received (%s), syntax error", config.tcpremoteip);
        lmprintf("555 syntax error (#5.5.4)\r\n");
        return (-1);
    }

    /* Increment the number of rcpts that we've received */
    rcpt_count++;

    typestr = NULL;
    local = 0;
    type = findRcptType(rcptaddr);
    switch (type) {
        case RCPT_REMOTE: 
            typestr = "remote";
            break;
        case RCPT_LOCAL_UNCHECKED:
            typestr = "local/unchecked";
            local = 1;
            break;
        case RCPT_LOCAL_NOEXIST:
            typestr = "local/noexist";
            local = 1;
            break;
        case RCPT_LOCAL_ERROR:
            typestr = "local/error";
            local = 1;
            break;
        case RCPT_LOCAL:
            typestr = "local/exist";
            local = 1;
            break;
        default:
            syslog(LOG_ERR, "Unknown rcpt_type %s:%s", (char *)__FILE__, (char *)__LINE__);
            break;
    }

    syslog(LOG_INFO, "RCPT command received (%s), args: %s (%s)"
                   , config.tcpremoteip
                   , LM_STRING_BUFFER(args), typestr ? typestr : "error");

    /* If the sender was in the bad mail from list, print an error mesage and return */
    if (bad_mail_from_flag) {
        syslog(LOG_NOTICE, "Rejecting RCPT. MAIL FROM in badmailfrom file.");
        lmprintf("553 sorry, your envelope sender is in my badmailfrom list (#5.7.1)\r\n");
        return (-1);
    }

    /* Check how many rcpt to's we've gotten and return error if 
     *    over the max value given */
    if ((config.max_rcpt > 0) && (rcpt_count > config.max_rcpt)) {
        syslog(LOG_NOTICE, "Rejecting RCPT. Too many recipients");

        /* print temp error message and continue */
        lmprintf("452 Too many recipients (#4.5.3.1)\r\n");

        /* Prevent an overflow from a very determined sender */
        rcpt_count = config.max_rcpt;
        return (-1);
    }

    /* Check how many rcpt's we've gotten and do a delay if over given value */
    if ((rcptnodelay != 1) && (config.rcpt_delay_at > 0) && (rcpt_count > config.rcpt_delay_at)) {
        if (rcpt_delay < config.rcpt_delay_max) {
            rcpt_delay += config.rcpt_delay_inc;
        }
        syslog(LOG_NOTICE, "Sleeping for %d seconds for RCPT delay", rcpt_delay);
        sleep(rcpt_delay);
    }


    LM_STRING_NEW(addr, LM_STRING_BUFFER(rcptaddr));

    if (!local) {
        if (config.tcprelayclient != NULL) {
            syslog(LOG_DEBUG, "RELAYCLIENT set. Appending [%s] to address", config.tcprelayclient);

            /* Append the value of relayclient after the address */
            LM_STRING_NEW(relayclient, config.tcprelayclient);
            if (LM_STRING_CAT(addr, relayclient)) {
                msd_error_out_of_memory();
                /* NOTREACHED */
            }
        } else {
            /* Return error so that the caller knows that this wasn't a good rcpt */
            lmprintf("553 sorry, that domain isn't in my list of allowed rcpthosts (#5.7.1)\r\n");
            return (-1);
        }
    }

    if (rcpt_bracket_ip_flag) {
        /*
         * Addresses in the form user@[X.X.X.X] always need to be accepted when
         * the IP matches one of ours
         *
         */
        return (0);
    }

    if (type == RCPT_LOCAL_NOEXIST) {
        /* Increment the number of non-existent user RCPTS */
        num_invalid_rcpt++;

        if ((config.max_invalid_rcpt <= num_invalid_rcpt) && (config.max_invalid_rcpt > 0)) {
            lmprintf("550 Too many invalid user RCPT's sent. Exiting.\r\n");
            syslog(LOG_NOTICE, "too many invalid rcpts: %d", num_invalid_rcpt);
            msd_quit_callback(1);
            /* NOTREACHED */
        }

        /* Output user does not exist */
        lmprintf("550 User does not exist\r\n");

        /* Log message about denying mail */
        syslog(LOG_INFO, "%s BLOCKED as non-valid user (%s)" , LM_STRING_BUFFER(addr), config.tcpremoteip);

        /* Return error so that rcpt counter is not incremented */
        return (-1);
    }

    /*
     * At this point, the address is local and either exists, or we're not 
     * sure.  So we try to load their spam rules.
     *
     */

    /* clean out any existing rules */
    cleanSpamRules(user_rules);

    /* Load the spam rules */

    stopwatch(&sw, STOPWATCH_START);
    retval = loadSpamRules(user_rules, LM_STRING_BUFFER(addr));
    if (retval == 0) {
        cleanSpamRules(global_rules);
        loadSpamRules(global_rules, "0");
    }
    syslog(LOG_DEBUG, "config_spam_rule_load() took %f seconds\n", stopwatch(&sw, STOPWATCH_STOP));

    syslog(LOG_DEBUG, "retval = %d", retval);

    // printSpamRules(user_rules);
    // printSpamRules(global_rules);

    if (retval == 0) {
        /* Spam rules loaded ok */

        /* disable rules that don't make sense if MAIL FROM is local */
        if (mailfrom_local_domain > 0) {
            syslog(LOG_DEBUG, "MAIL FROM is local, Disabling valid_bounce check");
            disableSpamRule(user_rules, SPAM_RULE_VALID_BOUNCE);
        }

        /* disable rules that don't make sense if RELAYCLIENT or AUTH */
        if ((config.tcprelayclient != NULL) || (authenticated_flag > 0)) {

            syslog(LOG_DEBUG, "relayclient set.  Disabling valid helo domain spam rule");
            disableSpamRule(user_rules, SPAM_RULE_VALID_HELO_DOMAIN);
            syslog(LOG_DEBUG, "relayclient set.  Disabling ip helo domain spam rule");
            disableSpamRule(user_rules, SPAM_RULE_IP_HELO_DOMAIN);
            syslog(LOG_DEBUG, "relayclient set.  Disabling resolve helo domain spam rule");
            disableSpamRule(user_rules, SPAM_RULE_RESOLVE_HELO_DOMAIN);
            syslog(LOG_DEBUG, "relayclient set.  Disabling dynamic reverse DNS check");
            disableSpamRule(user_rules, SPAM_RULE_CHECK_DYNAMIC_REVERSE_DNS);
            syslog(LOG_DEBUG, "relayclient set.  Disabling valid_bounce check");
            disableSpamRule(user_rules, SPAM_RULE_VALID_BOUNCE);
            syslog(LOG_DEBUG, "relayclient set.  Disabling require_helo check");
            disableSpamRule(user_rules, SPAM_RULE_REQUIRE_HELO);
            syslog(LOG_DEBUG, "relayclient set.  Disabling check_ip_reverse_dns check");
            disableSpamRule(user_rules, SPAM_RULE_CHECK_IP_REVERSE_DNS);

            /* BMS is disabled when relayclient is set in the spam_check.c file */
        }

#ifdef OPENSOURCE
	/* no quarantine in open source */
	enableSpamRule(user_rules, SPAM_RULE_SMTP_BLOCKING);	
#endif

        spam_rules_loaded_flag = 1;

        /* Check to see if the user has spam checking enabled */
        if (isEnabled(user_rules, SPAM_RULE_SPAM_CHECK)) {
            user = domain = NULL;

            if (get_user_domain(&user, &domain, addr) == 0) {
                initSpamLog(LM_STRING_BUFFER(user),
                            LM_STRING_BUFFER(domain),
                            LM_STRING_BUFFER(smtp_from_addr));
            } else {
                initSpamLog(NULL, NULL, LM_STRING_BUFFER(smtp_from_addr));
            }

            spam_check_retval = spam_check_do(user_rules, global_rules, addr, smtp_from_addr, &whitelist);
            writeSpamLog(NULL);

            /* XXX: MEMORY MAY BE LOST HERE */
        }
    } else {
        if (retval == -1) {
            /* There was an error loading the spam rules */
            syslog(LOG_ERR, "Error loading spam rules for '%s'\n", LM_STRING_BUFFER(addr));
            spam_check_retval = -1; /* Signify error */
        }
    }
    
    /* Set the global spam hit variable for the current message */
    spam_hit_flag = spam_check_retval > 0;

    /* NOTE: the order of the if..else if... stuff below is important */
    if (spam_check_retval == 137731) {
        syslog(LOG_NOTICE, "BMS BLOCK: any DATA received will not be delivered");
        lmprintf("550-User does not exist to you.\r\n");
        lmprintf("550-Your mail server [%s] is on one or more public block lists.  See:\r\n", config.tcpremoteip);
        lmprintf("550 http://www.linuxmagic.com/products/bms/lookup.php?address=%s\r\n",  config.tcpremoteip);
        retval = -1; 
    } else if (spam_check_retval <= 0) {
        /* either not spam, or an error */

    } else if (spam_rules_loaded_flag) {
        /* spam hit */
        
        if (isEnabled(user_rules, SPAM_RULE_SMTP_BLOCKING)) {
            syslog(LOG_NOTICE, "BLOCK: any DATA received will not be delivered");
            lmprintf("550 User does not exist to you\r\n");
            retval = -1; 
        } else {
            /* will mark for magic-local to quarantine */  
            retval = 0;
        }
    }

    if (spam_rules_loaded_flag) {
        cleanSpamRules(user_rules);
    }

    if ((retval >= 0) && (LM_STRING_CAT(smtp_rcpt_list, T) || LM_STRING_CAT(smtp_rcpt_list, addr))) {
        msd_error_out_of_memory();
    }

    return retval;
}


int msd_helo_callback(lm_string_t domain)
{
    /* Free the old helo host string and copy the arguents into a new one */
    LM_STRING_FREE(helohost);
    if (LM_STRING_DUP(helohost, domain) != 0) {
        msd_error_out_of_memory();
    }

    /* Set the fakehelohost variable to show that the given helo host doesn't match
       the one provided by tcp server */
    /* Returns 0 on match, fakehelohost set to non zero if they don't */
    fakehelohost = strncasecmp(config.tcpremotehost, LM_STRING_BUFFER(helohost), strlen(config.tcpremotehost) + 1);

    if (fakehelohost) {
        syslog(LOG_DEBUG, "HELO host does not match TCPREMOTEHOST");
    }

    lmprintf("250 %s\r\n", config.me);

    /* Return non-zero to stop default message */
    return -1;
}

int msd_ehlo_callback(lm_string_t domain)
{
    /* Free the old helo host string and copy the arguents into a new one */
    LM_STRING_FREE(helohost);
    if (LM_STRING_DUP(helohost, domain) != 0) {
        msd_error_out_of_memory();
    }

    /* Set the fakehelohost variable to show that the given helo host doesn't match
       the one provided by tcp server */
    /* Returns 0 on match, fakehelohost set to non zero if they don't */
    fakehelohost = strncasecmp(config.tcpremotehost, LM_STRING_BUFFER(helohost), strlen(config.tcpremotehost) + 1);

    if (fakehelohost) {
        syslog(LOG_DEBUG, "HELO host does not match TCPREMOTEHOST");
    }

    lmprintf("250-%s\r\n", config.me);
    if (config.auth_enable != 0) {
        lmprintf("250-AUTH LOGIN\r\n");
        lmprintf("250-AUTH=LOGIN\r\n");
    }
    lmprintf("250-PIPELINING\r\n");

#ifdef USE_TLS
    /* 
     * RFC 3207 says not to respond with STARTTLS in the extensions if it's
     * already being used.
     */
    if ((config.tls_enable == 1) && (using_tls == 0)) {
        lmprintf("250-STARTTLS\r\n");
    }
#endif

    lmprintf("250 8BITMIME\r\n");

    return 0;
}

/* Print a specific help string for this program */
int msd_help_callback(void)
{
    lmprintf("214 qmail home page: http://cr.yp.to/qmail.html, LinuxMagic Support http://www.linuxmagic.com\r\n");
    return 1;
}

#ifdef USE_TLS

int msd_starttls_callback(void)
{
    if ((config.tls_enable != 1) || (using_tls != 0)) {
        lmprintf("554 Error: TLS already active\r\n");
    } else {
        /*
         *    220 Ready to start TLS
         *    501 Syntax error (no parameters allowed)
         *    454 TLS not available due to temporary reason
         *
         */

        lmprintf("220 Ready to start TLS\r\n");

        /* RFC 3207 says reset the transaction here */
        /* XXX: does this reset the EHLO?  It needs to. */
        msd_rset_callback();

        if (initialize_ssl() == -1) {
            lmprintf("451 initialization failed\r\n");
            msd_quit_callback(1);
        }

        using_tls = 1;

        /* build this string to use with Recieved: header */
        if (asprintf(&encryption, "(%s encrypted) SMTP", SSL_get_cipher(ssl)) <= 0) {
            /* failed */
            encryption = NULL;
        }

        syslog(LOG_INFO, "CONNECTED using SSL");
    }
    return 1;
}

#endif


/* int msd_init(int argc, char **argv) */
int msd_init()
{
    int retval = 0;

    /* DISABLED by JW: setlogmask(LOG_UPTO(config.log_level)); */
    setlogmask(LOG_UPTO(7));

    /* default the helo string to "unknown" since this is what we compare
     * it to when we check whether it has been changed */
    LM_STRING_NEW(helohost, "unknown");

    fakehelohost = 0;    /* Real helo host given  */

    /* Put a log message in there */
    syslog(LOG_NOTICE, "Initializing for connection from %s", config.tcpremoteip);

    /* Allocate empty strings for data that can be received */
    if (LM_STRING_NEW(smtp_rcpt_list, "") || LM_STRING_NEW(smtp_from_addr, "")
        || LM_STRING_NEW(F, "F") || LM_STRING_NEW(T, "T") || LM_STRING_NEW(atsign, "@")
        || (((readbuf = malloc(config.max_line_length)) == NULL))) {
        msd_error_out_of_memory();
    }

    /* If anything failed in this we would have exited */
    return retval;
}

/* Just in case segfault handler so we know something happened */
void msd_sigsegfault(int arg)
{

    arg = arg;  /* Silence GCC warning */

    lmprintf("550 internal server error (segfault)\r\n");
    syslog(LOG_ERR, "internal server error, segfault");

    if (config.dump_core == 1) {
        if ((mkdir(segfaultdir, 0700) == 0) && (chdir(segfaultdir) == 0)) {
            /* abort so we can get a core dump */
            abort();
        } else {
            syslog(LOG_ERR, "unable to build dir for core dump");
        }
    }

    syslog(LOG_INFO, "Exiting");
    closelog();
    exit(1);
}

int main(int argc, char **argv)
{
    int retval;
    int numcmd = 0,smtpgreetlen;
    char compile_date[] = __DATE__;
    char compile_time[] = __TIME__;
    char features[1024];
    char *ptr, *smtpgreeting, *newline;
    int segfaultdirlen;
    struct rlimit rlim;
#ifndef OPENSOURCE
	char *tmp;
#endif

    /* setup for segfault core dump */
    segfaultdirlen = strlen(segfaultdir);
    if (segfaultdirlen > 5) {
        snprintf(segfaultdir + (segfaultdirlen - 5), 6, "%05d", (int)getpid());
    }
    rlim.rlim_cur = RLIM_INFINITY;
    rlim.rlim_max = RLIM_INFINITY;
    setrlimit(RLIMIT_CORE, &rlim);

    features[0] = '\0';

    /* this module cannot be disabled at this time */
    strcat(features, " +EXTPROG");

#ifdef USE_LM_DATABASE
    strcat(features, " +DATABASE");
#endif

#ifdef USE_LM_DBFILE
    strcat(features, " +DBFILE");
#endif

#ifdef USE_TLS
    strcat(features, " +TLS");
#endif

    if (argc == 2) {
        if (strncmp(argv[1], "-v", 2) == 0) {
            if (compile_date[4] == ' ') {
                compile_date[4] = '0';
            }                
            printf("Version: %s (compiled: %s %s)%s\n", MMVERSION_STRING,
                   compile_date, compile_time, features);
            exit(0);
        } else if (strncmp(argv[1], "-s", 2) == 0) {
            loadControls(10);
            exit(0);
        }
    }

    openlog("msd", LOG_PID, LOG_MAIL);
    setlogmask(LOG_UPTO(LOG_ERR));

    /* Disable pipe signals */
    signal(SIGPIPE, SIG_IGN);

    /* setup a segfault handler so we might get a message printed if we crash */
    signal(SIGSEGV, msd_sigsegfault);

    if (loadControls(0) != 0) {
        syslog(LOG_ERR, "load controls failed");
        msd_error_unavailable();
    }

    if (config.me == NULL) {
        syslog(LOG_ERR, "FATAL: qmail control \"me\" not set");
        msd_error_unavailable();
    }

    if (config.tcpremoteip == NULL) {
        config.tcpremoteip = "unknown";
    }

    if (config.tcpremotehost == NULL) {
        config.tcpremotehost = "unknown";
    }

#ifndef OPENSOURCE
    initMsdStatus();
#endif

    /* this will allow QHPSI variables to be set through controls */

#ifndef OPENSOURCE
    tmp = NULL;
    if ((config.qhpsi_enable == 1) && ((tmp = getenv("QHPSI")) == NULL)) {
        /* the user wants us to set qhpsi */
        if ((config.qhpsi_prog != NULL) && (strlen(config.qhpsi_prog) > 0)) {
	        syslog(LOG_DEBUG, "setting QHPSI = '%s'", config.qhpsi_prog);
            setenv("QHPSI", config.qhpsi_prog, 1);
            if ((config.qhpsi_max_size != NULL) && (strlen(config.qhpsi_max_size) > 0)) {
	            syslog(LOG_DEBUG, "setting QHPSIMAXSIZE = '%s'", config.qhpsi_max_size);
                setenv("QHPSIMAXSIZE", config.qhpsi_max_size, 1);
            }
            /* TODO: only if /usr/bin/clamdscan */
            if (getenv("QHPSIARG1") == NULL) {
                syslog(LOG_DEBUG, "setting QHPSIARG1 = '--disable-summary'");
                setenv("QHPSIARG1", "--disable-summary", 1);
            }
        }
    }

    if ((tmp != NULL) && (strlen(tmp) == 0)) {
	    syslog(LOG_DEBUG, "QHPSI set and empty, disabling");
	    unsetenv("QHPSI");
    }
#endif

    /* Initalize */
    if ((retval = msd_init()) != 0) {
        /* Little hack to keep the 221 quit message from being printed */
        syslog(LOG_ERR, "error initializing");
        msd_error_unavailable();
    }
    
    smtpgreeting = strdup(config.smtpgreeting);
    if (smtpgreeting == NULL) {
        syslog(LOG_ERR, "strdup(): %s", strerror(errno));
        msd_error_unavailable();
    }
    smtpgreetlen = strlen(smtpgreeting);

    /* NOTE: config_qmail trims trailing newlines */
    
    if ((newline = strchr(smtpgreeting, '\n')) == NULL) { 
        /* the smtpgreeting is one line, print it as is */
        lmprintf("220 %s ESMTP\r\n", smtpgreeting);
    } else {
        /* the smtpgreeting is multiple lines */

	    /* start with a header */
        lmprintf("220-%s ESMTP\r\n", config.me);

	    /* remove trailing newlines (should already be done) */
	    while (smtpgreeting[smtpgreetlen - 1] == '\n') {
	        smtpgreeting[--smtpgreetlen] = '\0';
	    }
    
	    ptr = smtpgreeting;
	    do {
	        *newline = '\0';
            lmprintf("220-%s\r\n", ptr);
	        *newline = '\n';
            ptr = newline + 1;
	    } while ((newline = strchr(ptr, '\n')) != NULL);
    
	    lmprintf("220 %s\r\n", ptr);
    }

    /* Read an smtp command from stdin */
    /* This function allocated the string to hold the command */
#ifdef USE_TLS
    if (smtp_init(config.timeoutsmtpd, msd_timeout_read_line_lm, msd_write,   /* read, write */
                  msd_queue_init, msd_queue, msd_queue_cleanup, msd_helo_callback, msd_ehlo_callback,   /* helo, ehlo */
                  msd_mail_callback, msd_rcpt_callback, /* mail, rcpt  */
                  msd_data_callback, msd_auth_callback, /* data, auth */
                  NULL, msd_rset_callback,  /* vrfy, rset */
                  msd_help_callback, NULL, msd_starttls_callback, /* help, noop, STARTTLS */
                  msd_quit_callback,    /* quit */
                  msd_header_line_callback, msd_body_line_callback) != 0) { /* header, body line callbacks */
#else 
    if (smtp_init(config.timeoutsmtpd, msd_timeout_read_line_lm, msd_write,   /* read, write */
                  msd_queue_init, msd_queue, msd_queue_cleanup, msd_helo_callback, msd_ehlo_callback,   /* helo, ehlo */
                  msd_mail_callback, msd_rcpt_callback, /* mail, rcpt  */
                  msd_data_callback, msd_auth_callback, /* data, auth */
                  NULL, msd_rset_callback,  /* vrfy, rset */
                  msd_help_callback, NULL, NULL, /* help, noop, STARTTLS */
                  msd_quit_callback,    /* quit */
                  msd_header_line_callback, msd_body_line_callback) != 0) { /* header, body line callbacks */
#endif
        syslog(LOG_ERR, "error initializing smtp");
        msd_error_unavailable();
    }

    if (getenv("RCPTNODELAY") != NULL) {
        rcptnodelay = 1;
    }

    /* Command loop */
    while (smtp_read_command(&smtp_cmd) >= 0) {
        if ((numcmd >= config.max_smtp_cmds) && (config.max_smtp_cmds > 0)) {
            lmprintf("552 Maximum number of SMTP commands per connection exceeded (#4.5.3.1)\r\n");
            syslog(LOG_WARNING, "maximum number of SMTP commands exceeded: %d", numcmd);
            msd_quit_callback(1);
        }

        /* Parse and handle the command, checking for errors */
        /* This function frees the string holding the commmand */
        if (smtp_handle_command(smtp_cmd) != 0) {
            /* QMAIL */
            /* TODO Print proper message here instead of what qmail does? */
            syslog(LOG_ERR, "error handling smtp command");
            msd_error_unavailable();
        }
        LM_STRING_FREE(smtp_cmd);
        numcmd++;
    }

    /* Exit the program */
    syslog(LOG_ERR, "error during smtp command processing...exiting");
    msd_error_unavailable();    /* doesn't return */
    return 0;
}

/* SIGALRM handler, prints timeout message and exits the program */
void msd_sigalrm(int arg)
{
    arg = arg;  /* Silence GCC warning */

    timeout_flag = 1;
    lmprintf("451 timeout (#4.4.2)\r\n");
    syslog(LOG_NOTICE, "read timeout");
    msd_quit_callback(1);
}

#define READ_BUFSIZE 1024


/*
 * Basic Procedure:
 *
 * 1  if data is left from last buffer and there is a whole line, return it
 * 2  otherwise, read some more data and append to buffer
 * 3  if timeout is reached, return indication of that fact
 * 4  if EOF is reached, return anything left in buffer and an indication
 * 5  goto 1
 *
 */
int msd_timeout_read_line(int fd, char **out, int *outlen)
{
    char buffer[READ_BUFSIZE], *new, *leftover;
    int timeout, n, i, newlen, leftoverlen;
    static int ptrlen = 0;
    static char *ptr = NULL;

    do {
        /* loop forever */

        if (ptr != NULL) {
            /* look for a '\n' character in the buffer */
            for (i = 0; i < ptrlen; i++) {
                if (ptr[i] == '\n') { 
                    /* the buffer contains a line, return that */

                    /* copy data to output string */
                    *outlen = i + 1;
                    *out = malloc(*outlen);
                    if (*out == NULL) {
                        msd_error_out_of_memory();
                    }
                    memcpy(*out, ptr, *outlen);

                    /* copy leftover data, so we can free current ptr */
                    leftoverlen = (ptrlen - *outlen);
                    leftover = malloc(leftoverlen);
                    if (leftover == NULL) {
                        msd_error_out_of_memory();
                    }
                    memcpy(leftover, (ptr + *outlen), leftoverlen);

                    /* the leftover from this run will be our ptr next time */
                    free(ptr);
                    ptr = leftover;
                    ptrlen = leftoverlen;

                    return (0);
                }
            }
        }

        /* if we get here, we need more data */

        n = msd_timeout_read(fd, buffer, sizeof(buffer) - 1, &timeout);
        if (n > 0) {
            /* got some characters, append to remaining data */

            /* first we need enough space for the new buffer */
            newlen = (ptrlen + n);
            new = malloc(newlen); 
            if (new == NULL) {
                msd_error_out_of_memory();
            }

            /* copy the data */
            if (ptrlen > 0) {
                memcpy(new, ptr, ptrlen);
            }
            memcpy((new + ptrlen), buffer, n);

            /* this will be used when we come back around the do-while() next time */
            ptrlen = newlen;
            if (ptr != NULL) {
                free(ptr);
            }
            ptr = new;
        } else {
            if (timeout == 1) {
                /* timed out in msd_timeout_read */
                syslog(LOG_ERR, "msd_timeout_read_line() timed out");
                lmprintf("451 timeout (#4.4.2)\r\n");
                msd_quit_callback(1);
                /* NOTREACHED */

            } else if (n == 0) {
                /* EOF, will be unable to send message so just exit */
                syslog(LOG_DEBUG, "EOF on stdin");
                msd_quit_callback(1);
                /* NOTREACHED */

            } else {
                /* XXX
                 *
                 * The cases where we can reach this need to be listed 
                 *
                 */
                syslog(LOG_INFO, "msd_timeout_read_line() internal error");
                lmprintf("451 internal error durring read\r\n");
                msd_quit_callback(1);
                /* NOTREACHED */
            }
        }
    } while(1);
    /* NOTREACHED */

    return (-1);
}

int msd_timeout_read(int fd, char *buf, int len, int *timeout)
{   
    fd_set rfds;
    struct timeval tv;

    *timeout = 0;


#ifdef USE_TLS 
    if ((fd == 0) && (using_tls == 1)) {
        /* we're using TLS and trying to read from STDIN, so use the TLS functions */

        if (SSL_pending(ssl) > 0) {
            /* data is already available */
            return (SSL_read(ssl, buf, len));
        }

        return (ssl_timeoutio(SSL_read, buf, len, timeout));
    }
#endif

    tv.tv_sec = config.timeoutsmtpd;
    tv.tv_usec = 0;
    FD_ZERO(&rfds);

    FD_SET(fd, &rfds);
    if (select(fd + 1, &rfds, (fd_set *) NULL, (fd_set *) NULL, &tv) == -1) {
        return (-1);
    }

    if (FD_ISSET(fd, &rfds)) {
        return (read(fd, buf, len));
    }

    /* timed out */
    *timeout = 1;

    return -1;
}


/* Write the given string to the file descriptor */
/* TODO Add timeouts and other error checking ? */
int msd_write(int fd, lm_string_t line)
{
    int retval;

    /* TODO should probably do some error checking here */
    LM_STRING_ASSERT(line);
    if (LM_STRING_CHECK(line) != 0) {
        return -1;
    }

    LM_ASSERT(fd > 0);
    retval = write(fd, LM_STRING_BUFFER(line), LM_STRING_LEN(line));

    /* TODO Do we want an fsync here all the time? */
    fsync(fd);
    return retval == LM_STRING_LEN(line);
}

#ifdef USE_LM_DATABASE

int msd_db_get_user_domain(char **pescuser, char **pescdomain,
                           lm_string_t useratdomain)
{
    lm_string_t username = NULL;
    lm_string_t domain = NULL;
    lm_string_t defaultdomain = NULL;
    int index;
    char *escuser = NULL;
    char *escdomain = NULL;
    char *buffer;

    if (get_user_domain(&username, &domain, useratdomain)) {
        msd_error_out_of_memory();
    }

    LM_STRING_NEW(defaultdomain, config.defaultdomain);

    if (LM_STRING_LEN(domain) == 0) {
        if (LM_STRING_CAT(domain, defaultdomain)) {
            msd_error_out_of_memory();
        }
    }

    /* HACK if sent to localhost address use qmail's default 
       domain to valid user checking */
    buffer = LM_STRING_BUFFER(domain);
    if (tolower(buffer[0]) == 'l' && tolower(buffer[1]) == 'o'
        && tolower(buffer[2]) == 'c' && tolower(buffer[3]) == 'a'
        && tolower(buffer[4]) == 'l' && tolower(buffer[5]) == 'h'
        && tolower(buffer[6]) == 'o' && tolower(buffer[7]) == 's'
        && tolower(buffer[8]) == 't' && buffer[9] == '\0') {
        LM_STRING_FREE(domain);
        if (LM_STRING_DUP(domain, defaultdomain)) {
            msd_error_out_of_memory();
        }
    }

    index = (LM_STRING_LEN(username) * 2) + 1;
    if ((escuser = malloc(index)) == NULL) {
        msd_error_out_of_memory();
    }
    index = (LM_STRING_LEN(domain) * 2) + 1;
    if ((escdomain = malloc(index)) == NULL) {
        msd_error_out_of_memory();
    }
    PQescapeString(escuser, LM_STRING_BUFFER(username), LM_STRING_LEN(username));
    PQescapeString(escdomain, LM_STRING_BUFFER(domain), LM_STRING_LEN(domain));
    LM_STRING_FREE(username);
    LM_STRING_FREE(domain);
    *pescuser = escuser;
    *pescdomain = escdomain;

    return 0;
}

#endif


int get_user(lm_string_t * puser, lm_string_t addr)
{
    int index;
    lm_string_t user = NULL;

    LM_STRING_DUP(user, addr);
    if (!LM_STRING_CHR(addr, '@', index)) {
        LM_STRING_END_SHIFT(user, LM_STRING_LEN(user) - index);
    }

    LM_ASSERT(puser);
    *puser = user;
    return *puser == NULL;
}

int get_domain(lm_string_t * pdomain, lm_string_t addr)
{
    int index;
    lm_string_t domain = NULL;

    if (!LM_STRING_CHR(addr, '@', index)) {
        LM_STRING_DUP(domain, addr);
        LM_STRING_START_SHIFT(domain, index + 1);
    } else {
        /* No @ sign, empty domain */
        LM_STRING_NEW(domain, "");
    }

    LM_ASSERT(pdomain != NULL);
    *pdomain = domain;
    return *pdomain == NULL;
}

/*  
  This function attempts to find the regular expression:
   [0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}
   which would match any dotted decimal ip address
   Returns 0 on sucess, non-zero on failure 
*/
int is_ip(lm_string_t addr)
{
    int retval = 0;
    int index = 0;
    int numdots = 0;
    int len;
    char *buffer;

    buffer = LM_STRING_BUFFER(addr);
    len = LM_STRING_LEN(addr);

    if (len == 0) {
        return 1;
    }

    while (!retval && index < len) {
        if (buffer[index] == '.') {
            numdots++;
            if (index > numdots * 3 + (numdots - 1)) {
                retval = 1;
            }
        } else {
            if (!isdigit(buffer[index])) {
                retval = 1;
            } else {
                if (index > (numdots + 1) * 3 + (numdots - 1)) {
                    retval = 1;
                }
            }
        }
        index++;
    }

    return retval;
}



int get_user_domain(lm_string_t * puser, lm_string_t * pdomain,
                    lm_string_t addr)
{
    int index;
    lm_string_t user = NULL;
    lm_string_t domain = NULL;

    LM_STRING_DUP(user, addr);
    if (!LM_STRING_CHR(user, '@', index)) {
        LM_STRING_DUP(domain, addr);
        LM_STRING_END_SHIFT(user, LM_STRING_LEN(user) - index);
        LM_STRING_START_SHIFT(domain, index + 1);
    } else {
        /* There was no @ specified so set the domain to nothing */
        LM_STRING_NEW(domain, "");
    }

    LM_ASSERT(puser != NULL);
    LM_ASSERT(pdomain != NULL);

    *puser = user;
    *pdomain = domain;

    return (*puser == NULL) || (*pdomain == NULL);
}
