/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 * Copyright (c) 2003-2005 LinuxMagic Inc. All Rights Reserved.
 *
 * This file contains Original Code and/or Modifications of Original Code as
 * defined in and that are subject to the Free Source Code License Version
 * 1.0 (the 'License'). You may not use this file except in compliance with
 * the License. Please obtain a copy of the License at:
 *
 * http://www.linuxmagic.com/opensource/licensing/FSCL.txt
 *
 * and read it before using this file.
 *
 * The Original Code and all software distributed under the License are
 * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESS OR IMPLIED, AND LINUXMAGIC HEREBY DISCLAIMS ALL SUCH WARRANTIES,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT. Please see
 * the License for the specific language governing rights and limitations
 * under the License."
 *
 * Please read the terms of this license carefully. By using or downloading
 * this software or file, you are accepting and agreeing to the terms of this
 * license with LinuxMagic Inc. If you are agreeing to this license on behalf
 * of a company, you represent that you are authorized to bind the company to
 * such a license. If you do not meet this criterion or you do not agree to
 * any of the terms of this license, do NOT download, distribute, use or alter
 * this software or file in any way.
 *
 * Author(s): Josh Wilsdon <josh@wizard.ca>
 *
 * CVS Id: $Header: /web/programmers/cvs/magicmail-backend/magic-smtpd/smtp_auth_dbfile.c,v 1.5 2005/12/22 00:50:41 josh Exp $
 *
 * DO NOT MODIFY WITHOUT CONSULTING THE LICENSE
 */

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <crypt.h>

#include <common/dbfile.h>
#include <common/controls.h>
#include "smtp_auth.h"
#include "smtp_auth_dbfile.h"

extern config_t config;
extern config_type_t config_type_info[];

int msd_do_auth_dbfile(int type, lm_string_t username, lm_string_t password)
{
    user_record_t user_rec;
    char *cryptpw;
    DB *dbp;
    int ret;
    lm_string_t at = NULL, tmpuser = NULL, defaultdomain = NULL;

    type = type;    /* remove GCC warning */

    dbp = openDBFile(config.user_info_dbfile);
    if (dbp == NULL) {
        /* TODO: syslog this higher up */
        return (LM_AUTH_ERROR);
    }

    if (strchr(LM_STRING_BUFFER(username), '@') == NULL) {
        /* append default domain if needed */
        syslog(LOG_DEBUG, "Username before default: [%s]\n",
               LM_STRING_BUFFER(username));
        LM_STRING_NEW(at, "@");
        LM_STRING_DUP(tmpuser, username);
        LM_STRING_CAT(tmpuser, at);
        LM_STRING_NEW(defaultdomain, config.defaultdomain);
        LM_STRING_CAT(tmpuser, defaultdomain);
        syslog(LOG_DEBUG, "Username after default: [%s]\n",
               LM_STRING_BUFFER(tmpuser));
        ret = getUserStruct(dbp, LM_STRING_BUFFER(tmpuser), &user_rec);
        LM_STRING_FREE(at);
        LM_STRING_FREE(tmpuser);
    } else {
        ret = getUserStruct(dbp, LM_STRING_BUFFER(username), &user_rec);
    }

    if (ret == 0) {
        /* got user data sucessfully */
	if ((user_rec.password == NULL) || (strlen(user_rec.password) < 1)) {
	    closeDBFile(dbp);
	    return (LM_AUTH_FAILED);
	}
        cryptpw = crypt(LM_STRING_BUFFER(password), user_rec.password);
        if (strncmp(cryptpw, user_rec.password, strlen(cryptpw)) == 0) {
            /* password matches */
            closeDBFile(dbp);
            return (LM_AUTH_OK);
        } else {
            closeDBFile(dbp);
            return (LM_AUTH_FAILED);
        }
    } else if (ret > 0) {
        /* user does not exist */
        closeDBFile(dbp);
        return (LM_AUTH_NO_USER);
    }

    closeDBFile(dbp);

    return (LM_AUTH_ERROR);
}

