/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 * Copyright (c) 2002-2005 LinuxMagic Inc. All Rights Reserved.
 *
 * This file contains Original Code and/or Modifications of Original Code as
 * defined in and that are subject to the Free Source Code License Version
 * 1.0 (the 'License'). You may not use this file except in compliance with
 * the License. Please obtain a copy of the License at:
 *
 * http://www.linuxmagic.com/opensource/licensing/
 *
 * and read it before using this file.
 *
 * The Original Code and all software distributed under the License are
 * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESS OR IMPLIED, AND LINUXMAGIC HEREBY DISCLAIMS ALL SUCH WARRANTIES,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT. Please see
 * the License for the specific language governing rights and limitations
 * under the License."
 *
 * Please read the terms of this license carefully. By using or downloading
 * this software or file, you are accepting and agreeing to the terms of this
 * license with LinuxMagic Inc. If you are agreeing to this license on behalf
 * of a company, you represent that you are authorized to bind the company to
 * such a license. If you do not meet this criterion or you do not agree to
 * any of the terms of this license, do NOT download, distribute, use or alter
 * this software or file in any way.
 *
 * Author(s): Burton Samograd <burton@wizard.ca>
 *            Josh Wilsdon <josh@wizard.ca>
 *
 * CVS Id: $Header: /web/programmers/cvs/magicmail-backend/magic-smtpd/spam_check.c,v 1.20 2005/12/22 00:50:41 josh Exp $
 *
 * DO NOT MODIFY WITHOUT CONSULTING THE LICENSE
 */

#include <assert.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <limits.h>
#include <liblm.h>

#include "magic-smtpd.h"
#include "spam_check.h"
#include "smtp_check.h"
#ifndef OPENSOURCE
#include "msdstatus.h"
#endif

//#include <utils.h>
#include <common/spam_log.h>
#include <common/controls.h>
#include <common/spam_rules.h>
#include <common/load_controls.h>
#include <common/ip2country.h>

#ifdef USE_BLOCK_LISTS
#include <blockmaps/lookup.h>
#endif

extern config_t config;
extern config_type_t config_type_info[];

#ifndef OPENSOURCE
extern msdstatus_t msdstatus;
#endif

extern int fakehelohost;
extern lm_string_t helohost;
extern int bounce_from;

int get_user_domain(lm_string_t * puser, lm_string_t * pdomain, lm_string_t addr);
int ip_to_country_code(lm_string_t * pcc, lm_string_t ip);


/* this is global defined in magic-smtpd.c */
extern int mail_from_rfc_formatted;

static lm_string_t fromuser = NULL;
static lm_string_t fromdomain = NULL;
static lm_string_t rcptuser = NULL;
static lm_string_t rcptdomain = NULL;
static lm_string_t fromcountry = NULL;
static lm_string_t helocountry = NULL;
static lm_string_t remoteipcountry = NULL;
static lm_string_t remoteip = NULL;
char *rcptemail = NULL;


int spam_check_init(lm_string_t rcptaddr, lm_string_t fromaddr, lm_string_t helodomain)
{
    int retval = 0;
    int index;

    if (get_user_domain(&rcptuser, &rcptdomain, rcptaddr)
        || get_user_domain(&fromuser, &fromdomain, fromaddr)) {
        retval = -1;
    }

    rcptemail = LM_STRING_BUFFER(rcptaddr);

    /* get the last part of the domain, which is possibly a country code */
    if (!LM_STRING_RCHR(fromdomain, '.', index)) {
        if (LM_STRING_NEW
            (fromcountry, LM_STRING_BUFFER(fromdomain) + index + 1)) {
            retval = 1;
        }
    } else {
        LM_STRING_NEW(fromcountry, "");
    }

    if (LM_STRING_LEN(helodomain) > 0) {
        /* get the last part of the domain, which is possibly a country code */
        if (!LM_STRING_RCHR(helodomain, '.', index)) {
            if (LM_STRING_NEW
                (helocountry, LM_STRING_BUFFER(helodomain) + index + 1)) {
                retval = 1;
            }                
        }
    } else {
        LM_STRING_NEW(helocountry, "");
    }

    LM_STRING_NEW(remoteip, config.tcpremoteip);
    if (ip_to_country_code(&remoteipcountry, remoteip))
        LM_STRING_NEW(remoteipcountry, "");
    /* If the country returned was not known ** is returned by the program. 
       Since we don't want to search the country list when this happens
       just turn that into an empty string so the search will automatically 
       succeed */
    if (LM_STRING_BUFFER(remoteipcountry)[0] == '*'
        && LM_STRING_BUFFER(remoteipcountry)[1] == '*') {
        LM_STRING_START_SHIFT(remoteipcountry, LM_STRING_LEN(remoteipcountry));
    }

    return retval;
}


void spam_check_cleanup(void)
{
    if (rcptuser) {
        LM_STRING_FREE(rcptuser);
    }
    if (rcptdomain) {
        LM_STRING_FREE(rcptdomain);
    }
    if (fromuser) {
        LM_STRING_FREE(fromuser);
    }
    if (fromdomain) {
        LM_STRING_FREE(fromdomain);
    }
    if (fromcountry) {
        LM_STRING_FREE(fromcountry);
    }
    if (helocountry) {
        LM_STRING_FREE(helocountry);
    }
    if (remoteipcountry) {
        LM_STRING_FREE(remoteipcountry);
    }
}


void noteSpam(char *rule, char *info, int score, enum hit_type global, unsigned int type)
{
    char typech;

    addSpamLog(rule, info, score, global);

    if (type == RULE_LIST) {
        if (score > 0) {
            typech = 'B';
        } else {
            typech = 'W';
        }
    } else {
        typech = 'S';
    }

#ifndef OPENSOURCE
    addMsdStatusEntry(rcptemail, typech, global, rule, info);
#endif

    return;
}


/* As much as I hate macros, these are way better than the old ones
 *
 * TODO this section should be refactored some more though when time permits
 *
 */ 
#define CHECK_RULE(RULE, FUNCTION, ARGS) \
syslog(LOG_DEBUG, "Checking rule: %s", #RULE); \
if (isEnabled(rules, RULE)) {   \
    if ((retval = FUNCTION ARGS ) != 0) { \
        syslog(LOG_INFO, "SPAM HIT: %s", #FUNCTION); \
        noteSpam(#FUNCTION, NULL, getScore(rules, RULE), (enum hit_type) USER, rules[RULE].type); \
        return (retval); \
    } \
}

#define CHECK_USER_LIST_REGEX(RULE, VALUE) \
if (VALUE != NULL) { \
    syslog(LOG_DEBUG, "Checking list: %s", #RULE); \
    if ((retval = checkListRegex(rules, RULE, LM_STRING_BUFFER(VALUE))) != 0) { \
        syslog(LOG_INFO, "SPAM HIT: %s", rules[RULE].key); \
        noteSpam(rules[RULE].key, NULL, getScore(rules, RULE), USER, rules[RULE].type); \
        if (getScore(rules, RULE) < 0) { \
            *whitelist = 1; \
            return (0); \
        } \
        return (retval); \
    } \
}

#define CHECK_GLOBAL_LIST_REGEX(RULE, VALUE) \
if (VALUE != NULL) { \
    syslog(LOG_DEBUG, "Checking list: %s", #RULE); \
    if ((retval = checkListRegex(global, RULE, LM_STRING_BUFFER(VALUE))) != 0) { \
        syslog(LOG_INFO, "SPAM HIT: %s", global[RULE].key); \
        noteSpam(global[RULE].key, NULL, getScore(global, RULE), GLOBAL, global[RULE].type); \
        if (getScore(global, RULE) < 0) { \
            *whitelist = 1; \
            return (0); \
        } \
        return (retval); \
    } \
}

#define CHECK_USER_LIST_STRCMP(RULE, VALUE) \
if (VALUE != NULL) { \
    syslog(LOG_DEBUG, "Checking list: %s", #RULE); \
    if ((retval = checkListStringCmp(rules, RULE, LM_STRING_BUFFER(VALUE))) != 0) { \
        syslog(LOG_INFO, "SPAM HIT: %s", rules[RULE].key); \
        noteSpam(rules[RULE].key, NULL, getScore(rules, RULE), USER, global[RULE].type); \
        if (getScore(rules, RULE) < 0) { \
            *whitelist = 1; \
            return (0); \
        } \
        return (retval); \
    } \
}

#define CHECK_GLOBAL_LIST_STRCMP(RULE, VALUE) \
if (VALUE != NULL) { \
    syslog(LOG_DEBUG, "Checking list: %s", #RULE); \
    if ((retval = checkListStringCmp(global, RULE, LM_STRING_BUFFER(VALUE))) != 0) { \
        syslog(LOG_INFO, "SPAM HIT: %s", global[RULE].key); \
        noteSpam(global[RULE].key, NULL, getScore(global, RULE), GLOBAL, global[RULE].type); \
        if (getScore(global, RULE) < 0) { \
            *whitelist = 1; \
            return (0); \
        } \
        return (retval); \
    } \
}


/* XXX Combine with addChar from magic-local.c and make into new library */
int addString(char **string, int *length, int *allocated, char *str)
{
    int len, orig_allocated;
    char *tmp;

    len = strlen(str);

    orig_allocated = *allocated;
    while (*allocated < (*length + len + 1)) {
        if (*allocated == 0) {
            *allocated = len + *length + 1;
        } else {
            *allocated *= 2;
        }
    }

    tmp = realloc(*string, *allocated);
    if (tmp == NULL) {
        *allocated = orig_allocated;
        syslog(LOG_ERR, "AHHHHH!!!!\n");
        return (1);
    }

    *string = tmp;

    while (*str != '\0') {
        (*string)[(*length)++] = *str;
        str++;
    }
    (*string)[(*length)] = '\0';

    return (0);
}


    
int spam_check_do(spam_rule_t * rules, spam_rule_t *global, lm_string_t rcptaddr, lm_string_t fromaddr, int *whitelist)
{
    int retval = 0;
    char hit_str[1024];

#ifdef USE_BLOCK_LISTS
	int block_list, found_bms, bms_allocated, bms_length;
    char *bms_str;
    spam_rule_entry_t *value;
    char oldpath[PATH_MAX];
#endif

    /* not found on whitelist yet */
    *whitelist = 0;

    /* no hits yet */
    hit_str[0] = '\0';

    assert(rules != NULL);
    LM_STRING_ASSERT(rcptaddr);
    LM_STRING_ASSERT(fromaddr);

    if (! isEnabled(rules, SPAM_RULE_SPAM_CHECK) || ! isEnabled(rules, SPAM_RULE_SMTP_CHECK)) {
        /* user does not want spam checking here */
        return (0);
    }

    if ((retval = spam_check_init(rcptaddr, fromaddr, helohost)) != 0) {
        /* if spam_check_init fails, return failure code */
        syslog(LOG_DEBUG, "spam_check_init: %d\n", retval);
        return (retval);
    }

    /*
     *
     * Do whitelist checks first, since we always need to do all of these 
     *
     */

    /* USER WHITELISTS */
    CHECK_USER_LIST_REGEX(SPAM_RULE_HELO_WHITELIST, helohost);
    CHECK_USER_LIST_REGEX(SPAM_RULE_FROM_WHITELIST, fromaddr);
    CHECK_USER_LIST_REGEX(SPAM_RULE_IP_WHITELIST, remoteip);

    /* GLOBAL WHITELISTS */
    if (isEnabled(rules, SPAM_RULE_USE_GLOBAL_HELO_WHITELIST)) {
        CHECK_GLOBAL_LIST_REGEX(SPAM_RULE_HELO_WHITELIST, helohost);
    }
    if (isEnabled(rules, SPAM_RULE_USE_GLOBAL_FROM_WHITELIST)) {
        CHECK_GLOBAL_LIST_REGEX(SPAM_RULE_FROM_WHITELIST, fromaddr);
    }
    if (isEnabled(rules, SPAM_RULE_USE_GLOBAL_IP_WHITELIST)) {
        CHECK_GLOBAL_LIST_REGEX(SPAM_RULE_IP_WHITELIST, remoteip);
    }

    /* check block lists */

#ifdef USE_BLOCK_LISTS

    bms_str = NULL;
    found_bms = bms_allocated = bms_length = 0;
    if ((getenv("RELAYCLIENT") == NULL) && (getenv("RBLSMTPD") == NULL)) {
        retval = block_list = 0;
        if (getcwd(oldpath, PATH_MAX) != NULL) {
	        if (chdir(config.block_list_dir) == 0) {
        	    /* user */
        	    value = rules[SPAM_RULE_BLOCK_LISTS].first;
        	    while (value != NULL) {
            	        syslog(LOG_DEBUG, "attempting block map lookup with file [%s]\n", (char *)value->data);
            	        block_list = lookupMapFile((char *)value->data, config.tcpremoteip);
            	        if (block_list == 1) {
                            if (bms_length > 0) {
                                addString(&bms_str, &bms_allocated, &bms_length, ",");
                            }
                            addString(&bms_str, &bms_allocated, &bms_length, "u");
                            addString(&bms_str, &bms_allocated, &bms_length, value->data);
                            found_bms = 1;
                            if (config.bms_honor_quarantine == 1) {
                                retval = 1;
                            } else {
                	            retval = 137731;    // magic number for magic feature
                            }
            	        } 
            	        value = value->next;
        	    }
	    
        	    /* global */
        	    if (isEnabled(rules, SPAM_RULE_USE_GLOBAL_BLOCK_LISTS)) {
            	        value = global[SPAM_RULE_BLOCK_LISTS].first;
            	        while (value != NULL) {
                	        syslog(LOG_DEBUG, "attempting block map lookup with file [%s]\n", (char *)value->data);
                	        block_list = lookupMapFile((char *)value->data, config.tcpremoteip);
                	        if (block_list == 1) {
                                if (bms_length > 0) {
                                    addString(&bms_str, &bms_allocated, &bms_length, ",");
                                }
                                addString(&bms_str, &bms_allocated, &bms_length, value->data);
                                found_bms = 1;
                                if (config.bms_honor_quarantine == 1) {
                                    retval = 1;
                                } else {
                    	            retval = 137731;    // magic number for magic feature
                                }
                 	        }
                	        value = value->next;
            	        }
        	    }
	    
        	    if (chdir(oldpath) != 0) {
                    syslog(LOG_ERR, "spam_check_do(): could not chdir back to oldpath");
        	    }
	        } else {
		        syslog(LOG_ERR, "lookupBlockLists() unable to chdir to list directory");
	        }
	    } else {
           syslog(LOG_ERR, "lookupBlockLists() unable to get current directory");
	    }
    }

    if (found_bms) {
        syslog(LOG_INFO, "SPAM HIT: block_lists: %s", bms_str);
        noteSpam(rules[SPAM_RULE_BLOCK_LISTS].key, bms_str, getScore(rules, SPAM_RULE_BLOCK_LISTS), USER, RULE_STRING);
    } 

    /* if (global_found_bms) {
        syslog(LOG_INFO, "SPAM HIT: global block_list: %s", (char *)value->data);
        noteSpam(global[SPAM_RULE_BLOCK_LISTS].key, (char *)value->data, getScore(global, SPAM_RULE_BLOCK_LISTS), GLOBAL, global[SPAM_RULE_BLOCK_LISTS].type);
    */

    if (retval != 0) {
        return (retval);
    }

#endif

    /* now do other checks */
    
    CHECK_RULE(SPAM_RULE_BLOCK_ALL_MAIL, block_all_mail, ());
    CHECK_RULE(SPAM_RULE_REQUIRE_FULL_ADDR, require_full_addr, (rcptaddr));
    CHECK_RULE(SPAM_RULE_BLOCK_MAIL_FROM_SELF, block_mail_from_self, (fromaddr, rcptaddr));
    CHECK_RULE(SPAM_RULE_REQUIRE_HELO, require_helo, (helohost));
    CHECK_RULE(SPAM_RULE_VALID_HELO_DOMAIN, valid_helo_domain, (helohost));
    CHECK_RULE(SPAM_RULE_IP_HELO_DOMAIN, ip_helo_domain, (helohost));
    CHECK_RULE(SPAM_RULE_RESOLVE_HELO_DOMAIN, resolve_helo_domain, (helohost));
    CHECK_RULE(SPAM_RULE_CHECK_IP_REVERSE_DNS, check_ip_reverse_dns, ());
    CHECK_RULE(SPAM_RULE_CHECK_DYNAMIC_REVERSE_DNS, check_dynamic_reverse_dns, ());
    CHECK_RULE(SPAM_RULE_BLOCK_IP_IN_ADDR, block_ip_in_addr, (rcptdomain));
    CHECK_RULE(SPAM_RULE_BLOCK_IP_IN_ADDR, block_ip_in_addr, (fromdomain));

    if ((LM_STRING_LEN(fromaddr) == 4) && (strcmp(LM_STRING_BUFFER(fromaddr), "#@[]") == 0)) {
        /* Special case, qmail uses #@[] as the double bounce MAIL FROM address */
        /* we do not want to check the domain on these messages */
        syslog(LOG_INFO, "Matched #@[], ignoring MAIL FROM checks.");
    } else if ((LM_STRING_LEN(fromaddr) == 0) && (mail_from_rfc_formatted == 1)) {
		/* this is a MAIL FROM: <> message, so a bounce */
		syslog(LOG_INFO, "MAIL FROM looks like a bounce, ignoring MAIL FROM checks.");
	} else {
        CHECK_RULE(SPAM_RULE_VALID_FROM_DOMAIN, valid_from_domain, (fromdomain));
        CHECK_RULE(SPAM_RULE_VALID_BOUNCE, valid_bounce, (fromdomain));
        CHECK_RULE(SPAM_RULE_MAIL_FROM_STRICT_ADDR_PARSE, mail_from_strict_addr_parse, (fromaddr, mail_from_rfc_formatted));
    }

    /*
     *
     * blacklist checks 
     *
     */

    /* USER BLACKLISTS */
    CHECK_USER_LIST_STRCMP(SPAM_RULE_COUNTRY_BLACKLIST, remoteipcountry);
    CHECK_USER_LIST_STRCMP(SPAM_RULE_COUNTRY_BLACKLIST, helocountry);
    CHECK_USER_LIST_STRCMP(SPAM_RULE_COUNTRY_BLACKLIST, fromcountry);
    CHECK_USER_LIST_REGEX(SPAM_RULE_HELO_BLACKLIST, helohost);
    CHECK_USER_LIST_REGEX(SPAM_RULE_FROM_BLACKLIST, fromaddr);
    CHECK_USER_LIST_REGEX(SPAM_RULE_IP_BLACKLIST, remoteip);

    /* GLOBAL BLACKLISTS */
    if (isEnabled(rules, SPAM_RULE_USE_GLOBAL_COUNTRY_BLACKLIST)) {
        CHECK_GLOBAL_LIST_STRCMP(SPAM_RULE_COUNTRY_BLACKLIST, remoteipcountry);
        CHECK_GLOBAL_LIST_STRCMP(SPAM_RULE_COUNTRY_BLACKLIST, helocountry);
        CHECK_GLOBAL_LIST_STRCMP(SPAM_RULE_COUNTRY_BLACKLIST, fromcountry);
    }
    if (isEnabled(rules, SPAM_RULE_USE_GLOBAL_HELO_BLACKLIST)) {
        CHECK_GLOBAL_LIST_REGEX(SPAM_RULE_HELO_BLACKLIST, helohost);
    }
    if (isEnabled(rules, SPAM_RULE_USE_GLOBAL_FROM_BLACKLIST)) {
        CHECK_GLOBAL_LIST_REGEX(SPAM_RULE_FROM_BLACKLIST, fromaddr);
    }
    if (isEnabled(rules, SPAM_RULE_USE_GLOBAL_IP_BLACKLIST)) {
        CHECK_GLOBAL_LIST_REGEX(SPAM_RULE_IP_BLACKLIST, remoteip);
    }

    spam_check_cleanup();

    return retval;
}
