/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 * Copyright (c) 2002-2005 LinuxMagic Inc. All Rights Reserved.
 *
 * This file contains Original Code and/or Modifications of Original Code as
 * defined in and that are subject to the Free Source Code License Version
 * 1.0 (the 'License'). You may not use this file except in compliance with
 * the License. Please obtain a copy of the License at:
 *
 * http://www.linuxmagic.com/opensource/licensing/FSCL.txt
 *
 * and read it before using this file.
 *
 * The Original Code and all software distributed under the License are
 * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESS OR IMPLIED, AND LINUXMAGIC HEREBY DISCLAIMS ALL SUCH WARRANTIES,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT. Please see
 * the License for the specific language governing rights and limitations
 * under the License."
 *
 * Please read the terms of this license carefully. By using or downloading
 * this software or file, you are accepting and agreeing to the terms of this
 * license with LinuxMagic Inc. If you are agreeing to this license on behalf
 * of a company, you represent that you are authorized to bind the company to
 * such a license. If you do not meet this criterion or you do not agree to
 * any of the terms of this license, do NOT download, distribute, use or alter
 * this software or file in any way.
 *
 * Author(s): Burton Samograd <burton@wizard.ca>
 *            Josh Wilsdon <josh@wizard.ca>
 *
 * CVS Id: $Header: /web/programmers/cvs/magicmail-backend/magic-smtpd/valid_user_check.c,v 1.3 2005/12/19 21:13:49 josh Exp $
 *
 * DO NOT MODIFY WITHOUT CONSULTING THE LICENSE
 */

#include <ctype.h>
#include <string.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>
#include <errno.h>


/* lmlib includes */
#include <liblm.h>

/* Application includes */
#include "valid_user_check.h"
#include "smtp.h"

#include <common/controls.h>
#include <common/load_controls.h>

extern config_t config;
extern config_type_t config_type_info[];

int msd_check_rcpt_user(lm_string_t * paddr)
{
    char *buffer;
    char postmaster[] = "postmaster";
    char postmasterat[] = "postmaster@";
    int ret = -1;

    /* look for postmaster address */
    buffer = LM_STRING_BUFFER(*paddr);
    if (strlen(buffer) == strlen(postmaster)) {
        if (strncasecmp(postmaster, buffer, strlen(postmaster)) == 0) {
            return (0);
        }
    } else if (strncasecmp(postmasterat, buffer, strlen(postmasterat)) == 0) {
        return (0);
    }

    /* not to postmaster or postmaster@ */

#ifdef USE_LM_DATABASE

    if (config.use_database == 1) {
        if ((ret = msd_check_rcpt_user_db(paddr)) != -1) {
            /* did not have an error so it did it's job */
            return (ret);
        }

        /* database check failed, so fall through to next method */
    }

#endif 

    /* CHECK USING EXTERNAL PROGRAM IF IT IS SET */

    if (config.ext_check_user_prog != NULL) {
        if ((ret = msd_check_rcpt_user_ext(LM_STRING_BUFFER(*paddr))) != -1) {
            /* check_user program did not have an error so it did it's job */
            return (ret);
        } 

        /* check_user program failed, so fall through to normal method */
    }

#ifdef USE_LM_DBFILE

    if (config.use_dbfile == 1) {
        if ((ret = msd_check_rcpt_user_dbfile(paddr)) != -1) {
            /* did not have an error so it did it's job */
            return (ret);
        }

        /* check failed, so fall through to next method */
    }

#endif

    return (ret);
}


