/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 * Copyright (c) 2002-2005 LinuxMagic Inc. All Rights Reserved.
 *
 * This file contains Original Code and/or Modifications of Original Code as
 * defined in and that are subject to the Free Source Code License Version
 * 1.0 (the 'License'). You may not use this file except in compliance with
 * the License. Please obtain a copy of the License at:
 *
 * http://www.linuxmagic.com/opensource/licensing/FSCL.txt
 *
 * and read it before using this file.
 *
 * The Original Code and all software distributed under the License are
 * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESS OR IMPLIED, AND LINUXMAGIC HEREBY DISCLAIMS ALL SUCH WARRANTIES,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT. Please see
 * the License for the specific language governing rights and limitations
 * under the License."
 *
 * Please read the terms of this license carefully. By using or downloading
 * this software or file, you are accepting and agreeing to the terms of this
 * license with LinuxMagic Inc. If you are agreeing to this license on behalf
 * of a company, you represent that you are authorized to bind the company to
 * such a license. If you do not meet this criterion or you do not agree to
 * any of the terms of this license, do NOT download, distribute, use or alter
 * this software or file in any way.
 *
 * Author(s): Burton Samograd <burton@wizard.ca>
 *            Josh Wilsdon <josh@wizard.ca>
 *
 * CVS Id: $Header: /web/programmers/cvs/magicmail-backend/magic-smtpd/valid_user_check_ext.c,v 1.4 2005/12/19 21:13:49 josh Exp $
 *
 * DO NOT MODIFY WITHOUT CONSULTING THE LICENSE
 */

#include <ctype.h>
#include <string.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>
#include <errno.h>

/* Application includes */
#include "valid_user_check.h"
#include "smtp.h"

#include <common/controls.h>
#include <common/load_controls.h>

extern config_t config;
extern config_type_t config_type_info[];

int msd_check_rcpt_user_ext(char *email)
{
    int retval, wpret;
    char childret;
    int childpid;

    char *prog;
    char *args[] = {
        NULL, NULL, NULL
    };

    /* Default return value for this function is user valid, as
     * If no program is specified, we already know we matched a -default
     * file (or else this function wouldn't have been called).  This function
     * just provides an extra level for virtual domain hosters to check a
     * database or whatever they want.
     */
    retval = 0;

    switch (childpid = fork()) {
        case 0:
            /* Launch the program in the child process */
            prog = config.ext_check_user_prog;
            args[0] = prog;
            args[1] = email;

            syslog(LOG_DEBUG, "running external check user program: [%s] [%s]\n", prog, email);

            if (execvp(prog, args)) {
                syslog(LOG_ERR, "external check_user program exec error: %s", strerror(errno));
            }

            _exit(-1);
            break;

        case -1:
            syslog(LOG_ERR, "external check_user program fork error: %s", strerror(errno));
            return(-1);
    }

    /* PARENT */

    /* Wait for the child to finish */
    do {
        wpret = waitpid(childpid, &retval, 0);
    } while ((wpret == -1) && (errno == EINTR));

    if (wpret != childpid) {
        /* Something bad happened */
        return -1;
    }

    /* Doing this to be safe.  WEXTISTATUS returns 0-255, which won't
     * do if our return value is an int.  Just be sure that it gets
     * sign extended by saving the exit status to a char and then
     * implicitly casting it to an int with the next assign
     */
    childret = WEXITSTATUS(retval);
    retval = childret;

    return retval;
}
