/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 * 
 * Copyright (c) 2002-2005 LinuxMagic Inc. All Rights Reserved.
 *
 * This file contains Original Code and/or Modifications of Original Code as
 * defined in and that are subject to the Free Source Code License Version
 * 1.0 (the 'License'). You may not use this file except in compliance with
 * the License. Please obtain a copy of the License at:
 *
 * http://www.linuxmagic.com/opensource/licensing/FSCL.txt
 *
 * and read it before using this file.
 *
 * The Original Code and all software distributed under the License are
 * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESS OR IMPLIED, AND LINUXMAGIC HEREBY DISCLAIMS ALL SUCH WARRANTIES,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT. Please see
 * the License for the specific language governing rights and limitations
 * under the License."
 *
 * Please read the terms of this license carefully. By using or downloading
 * this software or file, you are accepting and agreeing to the terms of this
 * license with LinuxMagic Inc. If you are agreeing to this license on behalf
 * of a company, you represent that you are authorized to bind the company to
 * such a license. If you do not meet this criterion or you do not agree to
 * any of the terms of this license, do NOT download, distribute, use or alter
 * this software or file in any way.
 *
 * Author(s): Josh Wilsdon <josh@wizard.ca>
 *
 * CVS Id: $Header: /web/programmers/cvs/magicmail-backend/spamrules/spamdir.c,v 1.4 2006/01/03 00:41:56 josh Exp $
 *
 * DO NOT MODIFY WITHOUT CONSULTING THE LICENSE
 */


#include <sys/types.h>
#include <sys/stat.h>

#include <dirent.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

/* where is this prototyped with Debian Unstable!? */
int asprintf(char **strp, const char *fmt, ...);

int getRules(void); 
int readFile(char *name);

int main(int argc, char **argv)
{
    struct stat buf;
    char *dir;

    if (argc != 2) {
        fprintf(stderr, "Usage: spamdir <email@domain>\n");
        exit(1);
    }

    /* disallow '/' characters, and don't let start w/ '.' */
    if ((strchr(argv[1], '/') != NULL) || (argv[1][0] == '.')) {
        // fprintf(stderr, "Fatal: invalid characters in email address\n");
        exit(1);
    }

    /* build directory string */
    if (asprintf(&dir, "/etc/magic-mail/spam_rules/%s", argv[1]) <= 0) {
        // fprintf(stderr, "Fatal: unable to build directory string: %s\n", strerror(errno));
        exit(1);
    }

    /* ensure dir is a directory */
    if (stat(dir, &buf) == 0) {
        if (S_ISDIR(buf.st_mode)) {
            if (chdir(dir) == 0) {
                /* read any rules and output to stdout */
                getRules(); 
                exit(0);
            } else {
                // fprintf(stderr, "Unable to chdir: %s\n", strerror(errno));
                exit(1);
            }
        }
    }

    // fprintf(stderr, "No such directory.\n");
    exit(1);
}

int getRules()
{
    DIR *dir;
    struct dirent *cur;

    dir = opendir(".");
    if (dir == NULL) {
        // fprintf(stderr, "Unable to open directory: %s\n", strerror(errno));
        return (1);
    }

    while ((cur = readdir(dir))) {
        if (cur->d_type == DT_REG) {
            readFile(cur->d_name);
        }
    }

    return(0);
}

int readFile(char *name)
{
    FILE *fp;
    int ch;
    char line[1024 + 1];

    fp = fopen(name, "r");
    if (fp == NULL) {
        return (-1);
    }

    while (fgets(line, sizeof(line), fp) != NULL) {
        if (line[strlen(line) - 1] == '\n') {
            line[strlen(line) - 1] = '\0';
            if (strlen(line) > 0) {
                printf("%s\n%s\n", name, line);
            }
        } else {
            /* XXX IGNORE THIS LINE, IT WAS TOO LONG */
            do {
                ch = fgetc(fp);
            } while ((ch != EOF) && (ch != '\n'));
        }
    }

    return (0);
}
