/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 * Copyright (c) 2004-2005 LinuxMagic Inc. All Rights Reserved.
 *
 * This file contains Original Code and/or Modifications of Original Code as
 * defined in and that are subject to the Free Source Code License Version
 * 1.0 (the 'License'). You may not use this file except in compliance with
 * the License. Please obtain a copy of the License at:
 *
 * http://www.linuxmagic.com/opensource/licensing/
 *
 * and read it before using this file.
 *
 * The Original Code and all software distributed under the License are
 * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESS OR IMPLIED, AND LINUXMAGIC HEREBY DISCLAIMS ALL SUCH WARRANTIES,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT. Please see
 * the License for the specific language governing rights and limitations
 * under the License."
 *
 * Please read the terms of this license carefully. By using or downloading
 * this software or file, you are accepting and agreeing to the terms of this
 * license with LinuxMagic Inc. If you are agreeing to this license on behalf
 * of a company, you represent that you are authorized to bind the company to
 * such a license. If you do not meet this criterion or you do not agree to
 * any of the terms of this license, do NOT download, distribute, use or alter
 * this software or file in any way.
 *
 * Author(s): Josh Wilsdon <josh@wizard.ca>
 *
 * CVS Id: $Id: spamgrep.c,v 1.5 2005/12/21 22:58:25 josh Exp $
 *
 * DO NOT MODIFY WITHOUT CONSULTING THE LICENSE
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <regex.h>

/* local includes */
#include <common/controls.h>
#include <common/spam_rules.h>
#include <common/load_controls.h>
 
#ifdef RELEASE_VERSION
#ifdef OPENSOURCE
    /* Opensource Release */
    #include <common/version.h.OPENSOURCE>
#else
    /* Commercial Release */
    #include <common/version.h.STABLE>
#endif
#else
    /* Development */
    #include <common/version.h>
#endif

/* globals */
config_t config;
config_type_t config_type_info[] = CONFIG_TYPE_DATA;
config_set_t config_set_info[] = CONFIG_SET_DATA;


int grepRules(spam_rule_t *rules, char *pattern);

int main(int argc, char **argv)
{
    char *email = NULL, *pattern = NULL;
    char features[] = "+EXTPROG +DATABASE +DBFILE";
    spam_rule_t user_rules[] = SPAM_RULES;
    int c, errors=0, sflag=0, vflag=0;
    extern char *optarg;
    extern int optind, optopt;
    enum backend_type type = NONE;

    while ((c = getopt(argc, argv, ":d:e:f:p:sv")) != -1) {
        switch(c) {
        case 'd':
            email = optarg;
            type = DATABASE;
            break;
        case 'e':
            email = optarg;
            type = EXT_PROG;
            break;
        case 'f':
            email = optarg;
            type = DBFILE;
            break;
        case 'p':
            pattern = optarg;
            break;
        case 'v':
            vflag = 1;
            break;
        case 's':
            sflag = 1;
            break;
        case ':':       /* -d -e -f without operand */
            errors++;
            fprintf(stderr, "Option -%c requires an operand\n", optopt);
            break;
        case '?':
            fprintf(stderr, "Unrecognized option: -%c\n", optopt);
            errors++;
            break;
        }
    }

    if (vflag == 1) {
        char compile_date[] = __DATE__;
        char compile_time[] = __TIME__;

        if (compile_date[4] == ' ') {
            compile_date[4] = '0';
        }
        printf("Version: %s (compiled: %s %s)%s\n", MMVERSION_STRING, compile_date, compile_time, features);
        exit(1);
    }

    if (sflag == 1) {
        loadControls(10);
        exit(1);
    }

    if ((errors > 0) || (type == NONE) || (pattern == NULL)) {
        fprintf(stderr, "Usage: spamgrep -[dfvs] <user@domain> -p <pattern>\n");
        exit(1);
    }

    if (loadControls(0) != 0) {
        /* failed to read config.  */
        fprintf(stderr, "FATAL: could not read configuration files\n");
        exit(1);
    }

    printf("grepping lists for [%s] using ", email);
    switch (type) {
        case DATABASE:
            printf("Database\n");
            break;
        case EXT_PROG:
            printf("External Program\n");
            break;
        case DBFILE:
            printf("DBFile\n");
            break;
        default:
            printf("NOTHING!\n");
            exit(1);
    }   

    loadSpamRulesHelper(user_rules, email, type);

    grepRules(user_rules, pattern);

    exit(0);
}



int grepRules(spam_rule_t *rules, char *pattern)
{
    spam_rule_t *ptr;
    spam_rule_entry_t *value;
    regex_t preg;
    char regerrbuf[80];
    int ret;

    ptr = rules;
    while (ptr->key != NULL) {
        if (ptr->entries > 0) {
	    if (ptr->type == RULE_LIST) {
	        // printf("%s(%d): ", ptr->key, ptr->entries);
	        value = ptr->first;
	        while ((value != NULL) && (value->data != NULL)) {
            	    /* regex */
                    ret = regcomp(&preg, (char *) value->data /* assumed to be regex */ , REG_ICASE | REG_EXTENDED | REG_NOSUB);
                    if (ret != 0) {
                        /* problem with regular expression */
                        regerror(ret, &preg, regerrbuf, sizeof(regerrbuf));
			printf("BADRE[%s]: %s\n", (char *) value->data, regerrbuf);
                    } else {
                        /* successfully compiled */
                        if (regexec(&preg, pattern, 0, NULL, 0) == 0) {
                            /* match */
			    printf("MATCH(%s)[%s]\n", ptr->key, (char *)value->data);
                        }
                    }
                    regfree(&preg);
	            value = value->next;
	        }
	    }
        }
        ptr++;
    }

    return (0);
}
