/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * Copyright (c) 2004-2005 LinuxMagic Inc. All Rights Reserved.
 *
 * This file contains Original Code and/or Modifications of Original Code as
 * defined in and that are subject to the Free Source Code License Version
 * 1.0 (the 'License'). You may not use this file except in compliance with
 * the License. Please obtain a copy of the License at:
 *
 * http://www.linuxmagic.com/opensource/licensing/
 *
 * and read it before using this file.
 *
 * The Original Code and all software distributed under the License are
 * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESS OR IMPLIED, AND LINUXMAGIC HEREBY DISCLAIMS ALL SUCH WARRANTIES,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT. Please see
 * the License for the specific language governing rights and limitations
 * under the License."
 *
 * Please read the terms of this license carefully. By using or downloading
 * this software or file, you are accepting and agreeing to the terms of this
 * license with LinuxMagic Inc. If you are agreeing to this license on behalf
 * of a company, you represent that you are authorized to bind the company to
 * such a license. If you do not meet this criterion or you do not agree to
 * any of the terms of this license, do NOT download, distribute, use or alter
 * this software or file in any way.
 *
 * Author(s): Josh Wilsdon <josh@wizard.ca>
 *
 * CVS Id: $Header: /web/programmers/cvs/magicmail-backend/spamrules/spamrules.c,v 1.9 2006/01/02 21:47:54 josh Exp $
 *
 * DO NOT MODIFY WITHOUT CONSULTING THE LICENSE
 */

#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

/* local includes */
#include <common/controls.h>
#include <common/spam_rules.h>
#include <common/load_controls.h>
 
#ifdef RELEASE_VERSION
#ifdef OPENSOURCE
    /* Opensource Release */
    #include <common/version.h.OPENSOURCE>
#else
    /* Commercial Release */
    #include <common/version.h.STABLE>
#endif
#else
    /* Development */
    #include <common/version.h>
#endif

/* globals */
config_t config;
config_type_t config_type_info[] = CONFIG_TYPE_DATA;
config_set_t config_set_info[] = CONFIG_SET_DATA;


int main(int argc, char **argv)
{
    char *email = NULL;
    char features[1024];
    spam_rule_t user_rules[] = SPAM_RULES;
    int c, errors=0, sflag=0, vflag=0;
    unsigned int i;
    extern char *optarg;
    extern int optind, optopt;
    enum backend_type type = NONE;

    features[0] = '\0';

    /* this module cannot be disabled at this time */
    strcat(features, " +EXTPROG");

#ifdef USE_LM_DATABASE
    strcat(features, " +DATABASE");
#endif

#ifdef USE_LM_DBFILE
    strcat(features, " +DBFILE");
#endif

    while ((c = getopt(argc, argv, ":d:e:f:sv")) != -1) {
        switch(c) {
        case 'd':
            email = optarg;
            type = DATABASE;
            break;
        case 'e':
            email = optarg;
            type = EXT_PROG;
            break;
        case 'f':
            email = optarg;
            type = DBFILE;
            break;
        case 'v':
            vflag = 1;
            break;
        case 's':
            sflag = 1;
            break;
        case ':':       /* -d -e -f without operand */
            errors++;
            fprintf(stderr, "Option -%c requires an operand\n", optopt);
            break;
        case '?':
            fprintf(stderr, "Unrecognized option: -%c\n", optopt);
            errors++;
            break;
        }
    }

    if (vflag == 1) {
        char compile_date[] = __DATE__;
        char compile_time[] = __TIME__;

        if (compile_date[4] == ' ') {
            compile_date[4] = '0';
        }
        printf("Version: %s (compiled: %s %s)%s\n", MMVERSION_STRING, compile_date, compile_time, features);
        exit(1);
    }

    if (sflag == 1) {
        loadControls(10);
        exit(1);
    }

    if ((errors > 0) || (type == NONE) || (email == NULL)) {
        fprintf(stderr, "Usage: spamrules -[vsdef] [<user@domain>]\n");
        exit(1);
    }

    if (loadControls(0) != 0) {
        /* failed to read config.  */
        fprintf(stderr, "FATAL: could not read configuration files\n");
        exit(1);
    }

	for (i=0; i < strlen(email); i++) {
		email[i] = tolower(email[i]);
	}

    printf("checking rules for [%s] using ", email);
    switch (type) {
        case DATABASE:
            printf("Database\n");
            break;
        case EXT_PROG:
            printf("External Program\n");
            break;
        case DBFILE:
            printf("DBFile\n");
            break;
        default:
            printf("NOTHING!\n");
            exit(1);
    }   

    loadSpamRulesHelper(user_rules, email, type);
    printSpamRules(user_rules);

    exit(0);
}


